searchData={"items":[{"type":"module","title":"ExRocketmq.Broker","doc":"RocketMQ Broker Client\n\nThe broker is the core component of RocketMQ. It handles responsibilities such as message storage,\nforwarding, and fulfilling consumer queries.\n\nEach broker cluster has a name server that handles registration and discovery.\nThe broker registers topic and queue metadata with the name server,\nand producers and consumers locate brokers through the name server to send and receive messages.\n\nWithin each broker there can be multiple queues, which store messages.\nFor a given topic, it can be deployed with multiple replicated queues across multiple brokers to ensure high availability.\n\nAdditionally, the broker offers message filtering and consumption balancing.\nMessage filtering permits only consuming a subset of messages,\nand consumption balancing can distribute messages among multiple consumer groups according to consumption speed.","ref":"ExRocketmq.Broker.html"},{"type":"function","title":"ExRocketmq.Broker.broker_name/1","doc":"","ref":"ExRocketmq.Broker.html#broker_name/1"},{"type":"function","title":"ExRocketmq.Broker.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExRocketmq.Broker.html#child_spec/1"},{"type":"function","title":"ExRocketmq.Broker.consumer_send_msg_back/2","doc":"When one consumer do consume jobs and failed many times, it will send a `consumer_send_msg_back` request to broker,\nthen broker will try to send the message to another consumer.","ref":"ExRocketmq.Broker.html#consumer_send_msg_back/2"},{"type":"function","title":"ExRocketmq.Broker.end_transaction/2","doc":"In the process of sending transactional messages, after the producer completes the local task check,\nit will send an end_transaction request to the broker to mark the completion of the transaction for that message.\n\nThe end_transaction request is a one-way request that may fail.\nIf the request fails, the broker will periodically send a check request to the producer\nto trigger the client to resend the end_transaction request.","ref":"ExRocketmq.Broker.html#end_transaction/2"},{"type":"function","title":"ExRocketmq.Broker.get_consumer_list_by_group/2","doc":"Get consumer list by group name.\nThis method is used in rebalance process.","ref":"ExRocketmq.Broker.html#get_consumer_list_by_group/2"},{"type":"function","title":"ExRocketmq.Broker.get_max_offset/2","doc":"Get last offset of queue in remote broker.","ref":"ExRocketmq.Broker.html#get_max_offset/2"},{"type":"function","title":"ExRocketmq.Broker.get_or_new_broker/5","doc":"In most cases, broker processes are started as subprocesses of producers or consumers.\nFor ease of management, they are generally started using dynamic_supervision,\nwhile also registering the addr as a key in the registry,\nmaking it possible to locate a specific broker process according to its addr.\n\nIf producer or consumer start broker process using `get_or_new_broker`,\nit must handle the `:notify` message sent by the broker process, like this:\n\n```Elixir\ndef handle_info({:notify, {pkt, broker_pid}} do\n  # handle pkt\nend\n```","ref":"ExRocketmq.Broker.html#get_or_new_broker/5"},{"type":"function","title":"Examples - ExRocketmq.Broker.get_or_new_broker/5","doc":"iex> ExRocketmq.Broker.get_or_new_broker(\"broker0\", \"localhost:31001\", RegistryName, DynamicSupervisorPid, nil)\n    #PID","ref":"ExRocketmq.Broker.html#get_or_new_broker/5-examples"},{"type":"function","title":"ExRocketmq.Broker.handle_continue/2","doc":"","ref":"ExRocketmq.Broker.html#handle_continue/2"},{"type":"function","title":"ExRocketmq.Broker.heartbeat/2","doc":"The heartbeat mechanism in RocketMQ is used to maintain the connection status between message producers and consumers and the message middleware.\nIt detects whether the connection is normal by periodically sending heartbeat packets and promptly identifies and handles connection exceptions.\n\nSpecifically, the heartbeat mechanism in RocketMQ has the following key points:\n\n- Sending heartbeat packets: Message producers and consumers regularly send heartbeat packets to the message middleware.\n  These packets contain necessary information such as the ID and version number of the producer/consumer.\n  This information helps the middleware identify the client sending the heartbeat.\n\n- Receiving heartbeat packets: The message middleware periodically receives heartbeat packets from producers and consumers.\n  By receiving these packets, the middleware can determine if the client is online and check the connection status.\n\n- Heartbeat timeout detection: The message middleware sets a heartbeat timeout period to check the reception of heartbeat packets.\n  If no heartbeat packet is received within the specified time, the middleware considers the client's connection to be abnormal and takes appropriate action.\n\n\nfor producer, heartbeat packet carries producer data set, for consumer, heartbeat packet carries consumer data set.","ref":"ExRocketmq.Broker.html#heartbeat/2"},{"type":"function","title":"Examples - ExRocketmq.Broker.heartbeat/2","doc":"iex> ExRocketmq.Broker.heartbeat(broker, %ExRocketmq.Models.Heartbeat{\n    ...>   client_id: \"producer_client_id\",\n    ...>   producer_data_set: [%ExRocketmq.Models.ProducerData{group: \"group\"}],\n    ...> })\n\n    iex> ExRocketmq.Broker.heartbeat(broker, %ExRocketmq.Models.Heartbeat{\n    ...>   client_id: \"consumer_client_id\",\n    ...>   consumer_data_set: [\n    ...>     %ExRocketmq.Models.ConsumerData{\n    ...>           group: \"group\"\n    ...>           consume_type: \"CONSUME_PASSIVELY\",\n    ...>           message_model: \"Clustering\",\n    ...>           consume_from_where: \"CONSUME_FROM_LAST_OFFSET\",\n    ...>           subscription_data_set: [\n    ...>             %ExRocketmq.Models.Subscription{\n    ...>               class_filter_mode: false,\n    ...>               topic: \"topic\",\n    ...>               sub_string: \"*\",\n    ...>               tags_set: [],\n    ...>               code_set: [],\n    ...>               sub_version: 0,\n    ...>               expression_type: \"TAG\"\n    ...>             }\n    ...>           ],\n    ...>           unit_mode: false\n    ...>     }\n    ...>   ],\n    ...> })","ref":"ExRocketmq.Broker.html#heartbeat/2-examples"},{"type":"function","title":"ExRocketmq.Broker.init/1","doc":"","ref":"ExRocketmq.Broker.html#init/1"},{"type":"function","title":"ExRocketmq.Broker.lock_batch_mq/2","doc":"In the scenario of sequential message consumption,\nthe consumer needs to exclusively acquire a certain message queue (mq).\nBy invoking this method, the consumer can perform a remote broker lock operation on the mq.","ref":"ExRocketmq.Broker.html#lock_batch_mq/2"},{"type":"function","title":"ExRocketmq.Broker.one_way_send_message/3","doc":"send message to broker and don't wait for response, the request is totally the same as `sync_send_message`,\nbroker actully will send response to client, but client ignore it.\nIt's kind of waste of network resource, maybe we can improve it in the future.","ref":"ExRocketmq.Broker.html#one_way_send_message/3"},{"type":"function","title":"ExRocketmq.Broker.pull_message/2","doc":"pull message from broker.\nThis request can also update queue's offset via `sys_flag` in `PullMsg.Request.t()`,\nso we can use it to update queue's offset while consuming message.","ref":"ExRocketmq.Broker.html#pull_message/2"},{"type":"function","title":"ExRocketmq.Broker.query_consumer_offset/2","doc":"query offset of queue in remote broker.\nIf the queue has never been consumed, a error code 22 will return","ref":"ExRocketmq.Broker.html#query_consumer_offset/2"},{"type":"function","title":"ExRocketmq.Broker.search_offset_by_timestamp/2","doc":"Search offset of queue by timestamp.\nThis method is used when consumer want to consume message from a specific timestamp.","ref":"ExRocketmq.Broker.html#search_offset_by_timestamp/2"},{"type":"function","title":"ExRocketmq.Broker.send_reply_pkt/2","doc":"","ref":"ExRocketmq.Broker.html#send_reply_pkt/2"},{"type":"function","title":"ExRocketmq.Broker.start_link/1","doc":"start a broker process, we use `get_or_new_broker` in most cases, `start_link` is used for special cases if needed.","ref":"ExRocketmq.Broker.html#start_link/1"},{"type":"function","title":"Parameters - ExRocketmq.Broker.start_link/1","doc":"* `:broker_name` (`t:String.t/0`) - Required. The name of the broker\n\n* `:remote_opts` (`t:term/0`) - Required. The remote options of the broker, see ExRocketmq.Remote for details\n\n* `:opts` (`t:keyword/0`) - The options for the broker The default value is `[]`.","ref":"ExRocketmq.Broker.html#start_link/1-parameters"},{"type":"function","title":"ExRocketmq.Broker.stop/1","doc":"","ref":"ExRocketmq.Broker.html#stop/1"},{"type":"function","title":"ExRocketmq.Broker.sync_send_message/3","doc":"send message to broker and wait for response.\nIf you want to send message asynchronously, you can wrap this method in a `Task`,\nso we don't provide a `async_send_message` method.","ref":"ExRocketmq.Broker.html#sync_send_message/3"},{"type":"function","title":"ExRocketmq.Broker.unlock_batch_mq/2","doc":"The reverse operation of lock_batch_mq is typically invoked\nafter rebalancing when the message queue (mq) is reassigned from one consumer to another.\nPrior to that, it is necessary to call this method to unlock the mq.","ref":"ExRocketmq.Broker.html#unlock_batch_mq/2"},{"type":"function","title":"ExRocketmq.Broker.update_consumer_offset/2","doc":"Set offset of queue in remote broker.\nIn most cases, we set remote offset using `pull_message` request.","ref":"ExRocketmq.Broker.html#update_consumer_offset/2"},{"type":"type","title":"ExRocketmq.Broker.namesrvs_opts_schema_t/0","doc":"","ref":"ExRocketmq.Broker.html#t:namesrvs_opts_schema_t/0"},{"type":"module","title":"ExRocketmq.Compress.Zlib","doc":"his module provides an implementation of the `ExRocketmq.Compressor` behaviour using the zlib library.\n\nThe `compress` function takes a binary `data` and compresses it using the zlib algorithm.\nIt accepts an optional `opts` keyword list that can be used to specify the compression level. The default level is `:best_compression`.\n\nThe `uncompress` function takes a binary `data` and decompresses it using the zlib algorithm.\n\nBoth functions return the compressed or uncompressed binary data, respectively.","ref":"ExRocketmq.Compress.Zlib.html"},{"type":"module","title":"Examples - ExRocketmq.Compress.Zlib","doc":"iex> compressed = ExRocketmq.Compress.Zlib.compress(\"hello world\")\n    iex> ExRocketmq.Compress.Zlib.uncompress(compressed)\n    \"hello world\"","ref":"ExRocketmq.Compress.Zlib.html#module-examples"},{"type":"behaviour","title":"ExRocketmq.Compressor","doc":"This module provides compression and decompression functionality for messages sent to and received from RocketMQ.\n\nIt defines a behavior `ExRocketmq.Compressor` that can be implemented by custom compression modules.","ref":"ExRocketmq.Compressor.html"},{"type":"behaviour","title":"Examples - ExRocketmq.Compressor","doc":"iex> ExRocketmq.Compressor.compress(MyCompressor, \"Hello World\")\n    \"compressed data\"","ref":"ExRocketmq.Compressor.html#module-examples"},{"type":"callback","title":"ExRocketmq.Compressor.compress/2","doc":"","ref":"ExRocketmq.Compressor.html#c:compress/2"},{"type":"function","title":"ExRocketmq.Compressor.compress/3","doc":"","ref":"ExRocketmq.Compressor.html#compress/3"},{"type":"callback","title":"ExRocketmq.Compressor.uncompress/1","doc":"","ref":"ExRocketmq.Compressor.html#c:uncompress/1"},{"type":"function","title":"ExRocketmq.Compressor.uncompress/2","doc":"","ref":"ExRocketmq.Compressor.html#uncompress/2"},{"type":"module","title":"ExRocketmq.Consumer","doc":"RocketMQ consumer\n\nThis module provides functionality for consuming messages from RocketMQ.\n\nIt defines a `State` struct that holds the state of the consumer and its configuration options.","ref":"ExRocketmq.Consumer.html"},{"type":"module","title":"Example - ExRocketmq.Consumer","doc":"1. Start a namesrvrs:\n  ```Elixir\n  {:ok, namesrvs_pid} = Namesrvs.start_link(\n    remotes: [\n     [transport: Transport.Tcp.new(host: \"test.rocket-mq.net\", port: 31_120)]\n   ],\n   opts: [\n     name: :namesrvs\n   ]\n  )\n  ```\n2. Define a processor:\n  ```Elixir\n  defmodule MyProcessor do\n\n    alias ExRocketmq.{\n      Consumer.Processor,\n      Models.MessageExt,\n      Typespecs\n    }\n\n    @behaviour Processor\n\n    defstruct []\n\n    @type t :: %__MODULE__{}\n\n    @spec new() :: t()\n    def new, do: %__MODULE__{}\n\n    @spec process(t(), Typespecs.topic(), [MessageExt.t()]) ::\n            Processor.consume_result() | {:error, term()}\n    def process(_, topic, msgs) do\n      msgs\n      |> Enum.each(fn msg ->\n        IO.inspect(msg)\n      end)\n\n      :success\n    end\n  end\n  ```\n\n3. Start consumer process\n   ```Elixir\n   Consumer.start_link(\n     consumer_group: \"GID_POETRY\",\n     namesrvs: :namesrvs,\n     processor: MyProcessor.new(),\n     subscriptions: %{\"POETRY\" => MsgSelector.new(:tag, \"*\")},\n     trace_enable: true,\n     opts: [\n       name: :consumer\n     ]}\n   )\n   ```","ref":"ExRocketmq.Consumer.html#module-example"},{"type":"function","title":"ExRocketmq.Consumer.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExRocketmq.Consumer.html#child_spec/1"},{"type":"function","title":"ExRocketmq.Consumer.handle_continue/2","doc":"","ref":"ExRocketmq.Consumer.html#handle_continue/2"},{"type":"function","title":"ExRocketmq.Consumer.init/1","doc":"","ref":"ExRocketmq.Consumer.html#init/1"},{"type":"function","title":"ExRocketmq.Consumer.start_link/1","doc":"Starts a RocketMQ consumer process.","ref":"ExRocketmq.Consumer.html#start_link/1"},{"type":"function","title":"Options - ExRocketmq.Consumer.start_link/1","doc":"* `:consumer_group` (`t:String.t/0`) - Required. Consumer group name\n\n* `:namespace` (`t:String.t/0`) - The namespace of the consumer group The default value is `\"\"`.\n\n* `:namesrvs` - Required. The namesrvs process\n\n* `:model` - Consumer model, :cluster or :broadcast The default value is `:cluster`.\n\n* `:subscriptions` (map of `t:String.t/0` keys and `t:term/0` values) - The subscriptions of the consumer, such as %{\"SomeTopic\" => %ExRocketmq.Models.MsgSelector{}} The default value is `%{}`.\n\n* `:consume_orderly` (`t:boolean/0`) - Whether to consume orderly The default value is `false`.\n\n* `:consume_from_where` - Where to start consuming, :last_offset or :first_offset The default value is `:last_offset`.\n\n* `:balance_strategy` (`t:term/0`) - The implemention of ExRocketmq.Consumer.BalanceStrategy The default value is `%ExRocketmq.Consumer.BalanceStrategy.Average{}`.\n\n* `:processor` (`t:term/0`) - Required. Message consume processor, must implement ExRocketmq.Consumer.Processor\n\n* `:consume_timestamp` (`t:non_neg_integer/0`) - The timestamp(ms) to consume from, only used when consume_from_where is :timestamp The default value is `0`.\n\n* `:post_subscription_when_pull` (`t:boolean/0`) - Whether to post subscription when pull message The default value is `false`.\n\n* `:pull_batch_size` (`t:non_neg_integer/0`) - The batch size to pull message The default value is `32`.\n\n* `:consume_batch_size` (`t:non_neg_integer/0`) - The batch size to consume message The default value is `16`.\n\n* `:max_reconsume_times` (`t:non_neg_integer/0`) - The max times to reconsume message The default value is `16`.\n\n* `:trace_enable` (`t:boolean/0`) - Whether to enable trace collection The default value is `false`.\n\n* `:opts` (`t:keyword/0`) - The opts of the comsumer's GenServer The default value is `[]`.","ref":"ExRocketmq.Consumer.html#start_link/1-options"},{"type":"function","title":"Examples - ExRocketmq.Consumer.start_link/1","doc":"iex> ExRocketmq.Consumer.start_link(%{group_name: \"my_group\", namesrvs: \"localhost:9876\"})\n    {:ok, pid}","ref":"ExRocketmq.Consumer.html#start_link/1-examples"},{"type":"function","title":"ExRocketmq.Consumer.stop/1","doc":"stop a RocketMQ consumer process.","ref":"ExRocketmq.Consumer.html#stop/1"},{"type":"function","title":"ExRocketmq.Consumer.subscribe/3","doc":"Subscribes to a RocketMQ topic with an optional message selector.\n\nAfter subscribing, the consumer will start to pull messages from the topic.\nAnd if the broker address changes, consumer update the connection to the broker.","ref":"ExRocketmq.Consumer.html#subscribe/3"},{"type":"function","title":"Examples - ExRocketmq.Consumer.subscribe/3","doc":"iex> ExRocketmq.Consumer.subscribe(consumer, \"my_topic\")\n    :ok","ref":"ExRocketmq.Consumer.html#subscribe/3-examples"},{"type":"function","title":"ExRocketmq.Consumer.unsubscribe/2","doc":"Unsubscribes from a RocketMQ topic.","ref":"ExRocketmq.Consumer.html#unsubscribe/2"},{"type":"function","title":"Examples - ExRocketmq.Consumer.unsubscribe/2","doc":"iex> ExRocketmq.Consumer.unsubscribe(consumer, \"my_topic\")\n    :ok","ref":"ExRocketmq.Consumer.html#unsubscribe/2-examples"},{"type":"type","title":"ExRocketmq.Consumer.consumer_opts_schema_t/0","doc":"","ref":"ExRocketmq.Consumer.html#t:consumer_opts_schema_t/0"},{"type":"behaviour","title":"ExRocketmq.Consumer.BalanceStrategy","doc":"The balance strategy behaviour of consumer","ref":"ExRocketmq.Consumer.BalanceStrategy.html"},{"type":"function","title":"ExRocketmq.Consumer.BalanceStrategy.allocate/4","doc":"Implementing a strategy to allocate an appropriate message queue for the current client","ref":"ExRocketmq.Consumer.BalanceStrategy.html#allocate/4"},{"type":"function","title":"Examples - ExRocketmq.Consumer.BalanceStrategy.allocate/4","doc":"iex> ExRocketmq.Consumer.BalanceStrategy.allocate(\n           ExRocketmq.Consumer.BalanceStrategy.Average.new(),\n           \"c1\",\n           [mq1, mq2, mq3],\n           [c1, c2, c3]\n         )\n    {:ok, [mq1]}","ref":"ExRocketmq.Consumer.BalanceStrategy.html#allocate/4-examples"},{"type":"callback","title":"ExRocketmq.Consumer.BalanceStrategy.allocate/4","doc":"","ref":"ExRocketmq.Consumer.BalanceStrategy.html#c:allocate/4"},{"type":"type","title":"ExRocketmq.Consumer.BalanceStrategy.t/0","doc":"","ref":"ExRocketmq.Consumer.BalanceStrategy.html#t:t/0"},{"type":"module","title":"ExRocketmq.Consumer.BalanceStrategy.Average","doc":"This module implements the balance strategy for RocketMQ consumers.\n\nThis strategy is called 'Average', which allocates the message queues (mq) evenly among the clients (cid).\nThe principle of this Average implementation is as follows:\n\n- Sort mqs and cid_all.\n- Divide mqs into len(cid_all) parts.\n- Allocate the corresponding chunk based on the position of cid in cid_all.","ref":"ExRocketmq.Consumer.BalanceStrategy.Average.html"},{"type":"module","title":"Usage - ExRocketmq.Consumer.BalanceStrategy.Average","doc":"1. Create a new instance of the strategy using `new/0` function.\n2. Call the `allocate/4` function to allocate message queues to a specific client.","ref":"ExRocketmq.Consumer.BalanceStrategy.Average.html#module-usage"},{"type":"module","title":"Example - ExRocketmq.Consumer.BalanceStrategy.Average","doc":"strategy = Average.new()\n    mq_all = [mq1, mq2, mq3, mq4]\n    cid_all = [\"client1\", \"client2\", \"client3\"]\n    {:ok, allocated_mqs} = Average.allocate(strategy, \"client2\", mq_all, cid_all)","ref":"ExRocketmq.Consumer.BalanceStrategy.Average.html#module-example"},{"type":"function","title":"ExRocketmq.Consumer.BalanceStrategy.Average.new/0","doc":"","ref":"ExRocketmq.Consumer.BalanceStrategy.Average.html#new/0"},{"type":"module","title":"ExRocketmq.Consumer.BuffManager","doc":"BuffManager is responsible for managing buffers for RocketMQ consumers.","ref":"ExRocketmq.Consumer.BuffManager.html"},{"type":"module","title":"Usage - ExRocketmq.Consumer.BuffManager","doc":"To use BuffManager, you need to start it using `start_link/1` function and provide a name:\n\n    {:ok, pid} = ExRocketmq.Consumer.BuffManager.start_link(name: :buff_manager)\n\nOnce started, you can use the `get_or_new/3` function to retrieve or create a buffer for a specific topic and queue ID.","ref":"ExRocketmq.Consumer.BuffManager.html#module-usage"},{"type":"module","title":"Example - ExRocketmq.Consumer.BuffManager","doc":"{:buff, offset, commit} = ExRocketmq.Consumer.BuffManager.get_or_new(:buff_manager, %MessageQueue{topic: \"my_topic\", queue_id: 1})","ref":"ExRocketmq.Consumer.BuffManager.html#module-example"},{"type":"function","title":"ExRocketmq.Consumer.BuffManager.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExRocketmq.Consumer.BuffManager.html#child_spec/1"},{"type":"function","title":"ExRocketmq.Consumer.BuffManager.delete_buff/2","doc":"","ref":"ExRocketmq.Consumer.BuffManager.html#delete_buff/2"},{"type":"function","title":"ExRocketmq.Consumer.BuffManager.get/2","doc":"","ref":"ExRocketmq.Consumer.BuffManager.html#get/2"},{"type":"function","title":"ExRocketmq.Consumer.BuffManager.get_or_new/2","doc":"","ref":"ExRocketmq.Consumer.BuffManager.html#get_or_new/2"},{"type":"function","title":"ExRocketmq.Consumer.BuffManager.init/1","doc":"","ref":"ExRocketmq.Consumer.BuffManager.html#init/1"},{"type":"function","title":"ExRocketmq.Consumer.BuffManager.start_link/1","doc":"","ref":"ExRocketmq.Consumer.BuffManager.html#start_link/1"},{"type":"function","title":"ExRocketmq.Consumer.BuffManager.stop/1","doc":"","ref":"ExRocketmq.Consumer.BuffManager.html#stop/1"},{"type":"function","title":"ExRocketmq.Consumer.BuffManager.update_offset/3","doc":"","ref":"ExRocketmq.Consumer.BuffManager.html#update_offset/3"},{"type":"type","title":"ExRocketmq.Consumer.BuffManager.t/0","doc":"","ref":"ExRocketmq.Consumer.BuffManager.html#t:t/0"},{"type":"behaviour","title":"ExRocketmq.Consumer.Processor","doc":"The processor is called when messages are received from the broker.","ref":"ExRocketmq.Consumer.Processor.html"},{"type":"function","title":"ExRocketmq.Consumer.Processor.process/3","doc":"","ref":"ExRocketmq.Consumer.Processor.html#process/3"},{"type":"callback","title":"ExRocketmq.Consumer.Processor.process/3","doc":"","ref":"ExRocketmq.Consumer.Processor.html#c:process/3"},{"type":"type","title":"ExRocketmq.Consumer.Processor.concurrent_consume_result/0","doc":"rocketmq provide 18 delay_level: 1s 5s 10s 30s 1m 2m 3m 4m 5m 6m 7m 8m 9m 10m 20m 30m 1h 2h\ndelay_level=1 means 1s, delay_level=2 means 5s and so on.","ref":"ExRocketmq.Consumer.Processor.html#t:concurrent_consume_result/0"},{"type":"type","title":"ExRocketmq.Consumer.Processor.consume_result/0","doc":"","ref":"ExRocketmq.Consumer.Processor.html#t:consume_result/0"},{"type":"type","title":"ExRocketmq.Consumer.Processor.orderly_consume_result/0","doc":"","ref":"ExRocketmq.Consumer.Processor.html#t:orderly_consume_result/0"},{"type":"type","title":"ExRocketmq.Consumer.Processor.t/0","doc":"","ref":"ExRocketmq.Consumer.Processor.html#t:t/0"},{"type":"module","title":"ExRocketmq.Models.BrokerClusterInfo","doc":"The broker cluster info model of rocketmq\n%{\n  \"brokerAddrTable\" => %{\n    \"sts-broker-d2-0\" => %{\n      \"brokerAddrs\" => %{\"0\" => \"10.88.4.57:20911\", \"1\" => \"10.88.4.189:20911\"},\n      \"brokerName\" => \"sts-broker-d2-0\",\n      \"cluster\" => \"d2\"\n    }\n  },\n  \"clusterAddrTable\" => %{\"d2\" => [\"sts-broker-d2-0\"]}\n}","ref":"ExRocketmq.Models.BrokerClusterInfo.html"},{"type":"function","title":"ExRocketmq.Models.BrokerClusterInfo.from_json/1","doc":"","ref":"ExRocketmq.Models.BrokerClusterInfo.html#from_json/1"},{"type":"function","title":"ExRocketmq.Models.BrokerClusterInfo.from_map/1","doc":"","ref":"ExRocketmq.Models.BrokerClusterInfo.html#from_map/1"},{"type":"type","title":"ExRocketmq.Models.BrokerClusterInfo.t/0","doc":"","ref":"ExRocketmq.Models.BrokerClusterInfo.html#t:t/0"},{"type":"module","title":"ExRocketmq.Models.BrokerData","doc":"The broker data model of rocketmq\n\n%{\n   \"brokerAddrs\" => %{\"0\" => \"10.88.4.78:20911\", \"1\" => \"10.88.4.144:20911\"},\n   \"brokerName\" => \"broker-0\",\n   \"cluster\" => \"d2\"\n }","ref":"ExRocketmq.Models.BrokerData.html"},{"type":"function","title":"ExRocketmq.Models.BrokerData.from_map/1","doc":"","ref":"ExRocketmq.Models.BrokerData.html#from_map/1"},{"type":"function","title":"ExRocketmq.Models.BrokerData.master_addr/1","doc":"","ref":"ExRocketmq.Models.BrokerData.html#master_addr/1"},{"type":"function","title":"ExRocketmq.Models.BrokerData.slave_addr/1","doc":"","ref":"ExRocketmq.Models.BrokerData.html#slave_addr/1"},{"type":"type","title":"ExRocketmq.Models.BrokerData.t/0","doc":"","ref":"ExRocketmq.Models.BrokerData.html#t:t/0"},{"type":"module","title":"ExRocketmq.Models.CheckTransactionState","doc":"The check transaction state header model","ref":"ExRocketmq.Models.CheckTransactionState.html"},{"type":"function","title":"ExRocketmq.Models.CheckTransactionState.decode/1","doc":"","ref":"ExRocketmq.Models.CheckTransactionState.html#decode/1"},{"type":"type","title":"ExRocketmq.Models.CheckTransactionState.t/0","doc":"","ref":"ExRocketmq.Models.CheckTransactionState.html#t:t/0"},{"type":"module","title":"ExRocketmq.Models.ConsumerSendMsgBack","doc":"consumer send message back model","ref":"ExRocketmq.Models.ConsumerSendMsgBack.html"},{"type":"type","title":"ExRocketmq.Models.ConsumerSendMsgBack.t/0","doc":"","ref":"ExRocketmq.Models.ConsumerSendMsgBack.html#t:t/0"},{"type":"module","title":"ExRocketmq.Models.EndTransaction","doc":"The end transaction header model","ref":"ExRocketmq.Models.EndTransaction.html"},{"type":"type","title":"ExRocketmq.Models.EndTransaction.t/0","doc":"","ref":"ExRocketmq.Models.EndTransaction.html#t:t/0"},{"type":"module","title":"ExRocketmq.Models.GetMaxOffset","doc":"get max offset model","ref":"ExRocketmq.Models.GetMaxOffset.html"},{"type":"type","title":"ExRocketmq.Models.GetMaxOffset.t/0","doc":"","ref":"ExRocketmq.Models.GetMaxOffset.html#t:t/0"},{"type":"module","title":"ExRocketmq.Models.Heartbeat","doc":"heartbeat data send to broker","ref":"ExRocketmq.Models.Heartbeat.html"},{"type":"function","title":"ExRocketmq.Models.Heartbeat.encode/1","doc":"","ref":"ExRocketmq.Models.Heartbeat.html#encode/1"},{"type":"function","title":"ExRocketmq.Models.Heartbeat.to_map/1","doc":"","ref":"ExRocketmq.Models.Heartbeat.html#to_map/1"},{"type":"type","title":"ExRocketmq.Models.Heartbeat.t/0","doc":"","ref":"ExRocketmq.Models.Heartbeat.html#t:t/0"},{"type":"module","title":"ExRocketmq.Models.Lock","doc":"","ref":"ExRocketmq.Models.Lock.html"},{"type":"module","title":"ExRocketmq.Models.Lock.Req","doc":"lock mq req","ref":"ExRocketmq.Models.Lock.Req.html"},{"type":"function","title":"ExRocketmq.Models.Lock.Req.encode/1","doc":"","ref":"ExRocketmq.Models.Lock.Req.html#encode/1"},{"type":"function","title":"ExRocketmq.Models.Lock.Req.to_map/1","doc":"","ref":"ExRocketmq.Models.Lock.Req.html#to_map/1"},{"type":"type","title":"ExRocketmq.Models.Lock.Req.t/0","doc":"","ref":"ExRocketmq.Models.Lock.Req.html#t:t/0"},{"type":"module","title":"ExRocketmq.Models.Lock.Resp","doc":"lock mq resp","ref":"ExRocketmq.Models.Lock.Resp.html"},{"type":"function","title":"ExRocketmq.Models.Lock.Resp.decode/1","doc":"","ref":"ExRocketmq.Models.Lock.Resp.html#decode/1"},{"type":"type","title":"ExRocketmq.Models.Lock.Resp.t/0","doc":"","ref":"ExRocketmq.Models.Lock.Resp.html#t:t/0"},{"type":"module","title":"ExRocketmq.Models.Message","doc":"The model of message sent by producer or consumed by consumer","ref":"ExRocketmq.Models.Message.html"},{"type":"function","title":"ExRocketmq.Models.Message.decode_properties/1","doc":"","ref":"ExRocketmq.Models.Message.html#decode_properties/1"},{"type":"function","title":"ExRocketmq.Models.Message.encode/1","doc":"","ref":"ExRocketmq.Models.Message.html#encode/1"},{"type":"function","title":"ExRocketmq.Models.Message.encode_properties/1","doc":"","ref":"ExRocketmq.Models.Message.html#encode_properties/1"},{"type":"function","title":"ExRocketmq.Models.Message.get_property/3","doc":"","ref":"ExRocketmq.Models.Message.html#get_property/3"},{"type":"function","title":"ExRocketmq.Models.Message.with_properties/2","doc":"","ref":"ExRocketmq.Models.Message.html#with_properties/2"},{"type":"function","title":"ExRocketmq.Models.Message.with_property/3","doc":"","ref":"ExRocketmq.Models.Message.html#with_property/3"},{"type":"type","title":"ExRocketmq.Models.Message.t/0","doc":"","ref":"ExRocketmq.Models.Message.html#t:t/0"},{"type":"module","title":"ExRocketmq.Models.MessageExt","doc":"extended message model","ref":"ExRocketmq.Models.MessageExt.html"},{"type":"function","title":"ExRocketmq.Models.MessageExt.decode_from_binary/1","doc":"","ref":"ExRocketmq.Models.MessageExt.html#decode_from_binary/1"},{"type":"function","title":"ExRocketmq.Models.MessageExt.get_offset_message_id/3","doc":"","ref":"ExRocketmq.Models.MessageExt.html#get_offset_message_id/3"},{"type":"type","title":"ExRocketmq.Models.MessageExt.t/0","doc":"","ref":"ExRocketmq.Models.MessageExt.html#t:t/0"},{"type":"module","title":"ExRocketmq.Models.MessageQueue","doc":"The mq info model of producer","ref":"ExRocketmq.Models.MessageQueue.html"},{"type":"function","title":"ExRocketmq.Models.MessageQueue.equal?/2","doc":"","ref":"ExRocketmq.Models.MessageQueue.html#equal?/2"},{"type":"function","title":"ExRocketmq.Models.MessageQueue.to_map/1","doc":"","ref":"ExRocketmq.Models.MessageQueue.html#to_map/1"},{"type":"type","title":"ExRocketmq.Models.MessageQueue.t/0","doc":"","ref":"ExRocketmq.Models.MessageQueue.html#t:t/0"},{"type":"module","title":"ExRocketmq.Models.MsgSelector","doc":"consumer message selector","ref":"ExRocketmq.Models.MsgSelector.html"},{"type":"function","title":"ExRocketmq.Models.MsgSelector.codes/1","doc":"","ref":"ExRocketmq.Models.MsgSelector.html#codes/1"},{"type":"function","title":"ExRocketmq.Models.MsgSelector.new/2","doc":"","ref":"ExRocketmq.Models.MsgSelector.html#new/2"},{"type":"function","title":"ExRocketmq.Models.MsgSelector.tags/1","doc":"","ref":"ExRocketmq.Models.MsgSelector.html#tags/1"},{"type":"type","title":"ExRocketmq.Models.MsgSelector.t/0","doc":"","ref":"ExRocketmq.Models.MsgSelector.html#t:t/0"},{"type":"module","title":"ExRocketmq.Models.PullMsg","doc":"request/response header model for pull message","ref":"ExRocketmq.Models.PullMsg.html"},{"type":"module","title":"ExRocketmq.Models.QueryConsumerOffset","doc":"query consumer offset model","ref":"ExRocketmq.Models.QueryConsumerOffset.html"},{"type":"type","title":"ExRocketmq.Models.QueryConsumerOffset.t/0","doc":"","ref":"ExRocketmq.Models.QueryConsumerOffset.html#t:t/0"},{"type":"module","title":"ExRocketmq.Models.QueueData","doc":"The queue data model of rocketmq\n%{\n   \"brokerName\" => \"broker-0\",\n   \"perm\" => 6,\n   \"readQueueNums\" => 2,\n   \"topicSynFlag\" => 0,\n   \"writeQueueNums\" => 2\n }","ref":"ExRocketmq.Models.QueueData.html"},{"type":"function","title":"ExRocketmq.Models.QueueData.from_map/1","doc":"","ref":"ExRocketmq.Models.QueueData.html#from_map/1"},{"type":"function","title":"ExRocketmq.Models.QueueData.readable?/1","doc":"","ref":"ExRocketmq.Models.QueueData.html#readable?/1"},{"type":"function","title":"ExRocketmq.Models.QueueData.to_consume_queues/2","doc":"","ref":"ExRocketmq.Models.QueueData.html#to_consume_queues/2"},{"type":"function","title":"ExRocketmq.Models.QueueData.to_publish_queues/2","doc":"","ref":"ExRocketmq.Models.QueueData.html#to_publish_queues/2"},{"type":"type","title":"ExRocketmq.Models.QueueData.t/0","doc":"","ref":"ExRocketmq.Models.QueueData.html#t:t/0"},{"type":"module","title":"ExRocketmq.Models.SearchOffset","doc":"search offset model","ref":"ExRocketmq.Models.SearchOffset.html"},{"type":"type","title":"ExRocketmq.Models.SearchOffset.t/0","doc":"","ref":"ExRocketmq.Models.SearchOffset.html#t:t/0"},{"type":"module","title":"ExRocketmq.Models.SendMsg","doc":"request/response header model for send message","ref":"ExRocketmq.Models.SendMsg.html"},{"type":"module","title":"ExRocketmq.Models.Subscription","doc":"subscription data model","ref":"ExRocketmq.Models.Subscription.html"},{"type":"function","title":"ExRocketmq.Models.Subscription.to_map/1","doc":"","ref":"ExRocketmq.Models.Subscription.html#to_map/1"},{"type":"type","title":"ExRocketmq.Models.Subscription.t/0","doc":"","ref":"ExRocketmq.Models.Subscription.html#t:t/0"},{"type":"module","title":"ExRocketmq.Models.TopicRouteInfo","doc":"The topic route info model of rocketmq\n\n%{\n   \"brokerDatas\" => [\n     %{\n       \"brokerAddrs\" => %{\"0\" => \"10.88.4.78:20911\", \"1\" => \"10.88.4.144:20911\"},\n       \"brokerName\" => \"broker-0\",\n       \"cluster\" => \"d2\"\n     }\n   ],\n   \"queueDatas\" => [\n     %{\n       \"brokerName\" => \"broker-0\",\n       \"perm\" => 6,\n       \"readQueueNums\" => 2,\n       \"topicSynFlag\" => 0,\n       \"writeQueueNums\" => 2\n     }\n   ]\n }","ref":"ExRocketmq.Models.TopicRouteInfo.html"},{"type":"function","title":"ExRocketmq.Models.TopicRouteInfo.from_json/1","doc":"","ref":"ExRocketmq.Models.TopicRouteInfo.html#from_json/1"},{"type":"function","title":"ExRocketmq.Models.TopicRouteInfo.from_map/1","doc":"","ref":"ExRocketmq.Models.TopicRouteInfo.html#from_map/1"},{"type":"type","title":"ExRocketmq.Models.TopicRouteInfo.t/0","doc":"","ref":"ExRocketmq.Models.TopicRouteInfo.html#t:t/0"},{"type":"module","title":"ExRocketmq.Models.Trace","doc":"trace data model","ref":"ExRocketmq.Models.Trace.html"},{"type":"function","title":"ExRocketmq.Models.Trace.encode/1","doc":"","ref":"ExRocketmq.Models.Trace.html#encode/1"},{"type":"type","title":"ExRocketmq.Models.Trace.t/0","doc":"","ref":"ExRocketmq.Models.Trace.html#t:t/0"},{"type":"module","title":"ExRocketmq.Models.TraceItem","doc":"trace item data model","ref":"ExRocketmq.Models.TraceItem.html"},{"type":"type","title":"ExRocketmq.Models.TraceItem.t/0","doc":"","ref":"ExRocketmq.Models.TraceItem.html#t:t/0"},{"type":"module","title":"ExRocketmq.Models.UpdateConsumerOffset","doc":"update consumer offset model","ref":"ExRocketmq.Models.UpdateConsumerOffset.html"},{"type":"type","title":"ExRocketmq.Models.UpdateConsumerOffset.t/0","doc":"","ref":"ExRocketmq.Models.UpdateConsumerOffset.html#t:t/0"},{"type":"module","title":"ExRocketmq.Namesrvs","doc":"The namesrvs layer of the rocketmq: how client communicates with the namesrvs.\nMost time, you should start nameserver under the supervision tree of your application.\n\nEG:\n```elixir\n\n    Supervisor.start_link(\n      [\n        {Namesrvs,\n         remotes: [\n           [transport: Transport.Tcp.new(host: \"test.rocket-mq.net\", port: 31_120)]\n         ],\n         opts: [\n           name: :namesrvs\n         ]},\n      ],\n      strategy: :one_for_one\n    )\n```\n\nYour consumer or producer module can communicate with namesrvs by its name","ref":"ExRocketmq.Namesrvs.html"},{"type":"function","title":"ExRocketmq.Namesrvs.addrs/1","doc":"","ref":"ExRocketmq.Namesrvs.html#addrs/1"},{"type":"function","title":"ExRocketmq.Namesrvs.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExRocketmq.Namesrvs.html#child_spec/1"},{"type":"function","title":"ExRocketmq.Namesrvs.get_broker_cluster_info/1","doc":"get the broker cluster info from the namesrvs","ref":"ExRocketmq.Namesrvs.html#get_broker_cluster_info/1"},{"type":"function","title":"Examples - ExRocketmq.Namesrvs.get_broker_cluster_info/1","doc":"iex> ExRocketmq.Namesrvs.get_broker_cluster_info(namesrvs)\n    {:ok,\n      %ExRocketmq.Models.BrokerClusterInfo{\n        broker_addr_table: %{\n          \"sts-broker-d2-0\" => %ExRocketmq.Models.BrokerData{\n            broker_addrs: %{\"0\" => \"10.88.4.57:20911\", \"1\" => \"10.88.4.189:20911\"},\n            broker_name: \"sts-broker-d2-0\",\n            cluster: \"d2\"\n          }\n        },\n        cluster_addr_table: %{\"d2\" => [\"sts-broker-d2-0\"]}\n    }}","ref":"ExRocketmq.Namesrvs.html#get_broker_cluster_info/1-examples"},{"type":"function","title":"ExRocketmq.Namesrvs.init/1","doc":"","ref":"ExRocketmq.Namesrvs.html#init/1"},{"type":"function","title":"ExRocketmq.Namesrvs.query_topic_route_info/2","doc":"query the topic route info from the namesrvs","ref":"ExRocketmq.Namesrvs.html#query_topic_route_info/2"},{"type":"function","title":"Examples - ExRocketmq.Namesrvs.query_topic_route_info/2","doc":"iex> ExRocketmq.Namesrvs.query_topic_route_info(namesrvs, \"topic\")\n    {:ok,\n     %ExRocketmq.Models.TopicRouteInfo{\n       broker_datas: [\n         %ExRocketmq.Models.BrokerData{\n           broker_addrs: %{\"0\" => \"10.88.4.78:20911\", \"2\" => \"10.88.4.144:20911\"},\n           broker_name: \"sts-broker-d2-0\",\n           cluster: \"d2\"\n         }\n       ],\n       queue_datas: [\n         %ExRocketmq.Models.QueueData{\n           broker_name: \"sts-broker-d2-0\",\n           perm: 6,\n           read_queue_nums: 2,\n           topic_syn_flag: 0,\n           write_queue_nums: 2\n         }\n       ]\n     }}","ref":"ExRocketmq.Namesrvs.html#query_topic_route_info/2-examples"},{"type":"function","title":"ExRocketmq.Namesrvs.start_link/1","doc":"","ref":"ExRocketmq.Namesrvs.html#start_link/1"},{"type":"function","title":"ExRocketmq.Namesrvs.stop/1","doc":"","ref":"ExRocketmq.Namesrvs.html#stop/1"},{"type":"type","title":"ExRocketmq.Namesrvs.namesrvs_opts_schema_t/0","doc":"","ref":"ExRocketmq.Namesrvs.html#t:namesrvs_opts_schema_t/0"},{"type":"behaviour","title":"ExRocketmq.OffsetStore","doc":"Rocketmq offset store, manage offset for consumer","ref":"ExRocketmq.OffsetStore.html"},{"type":"callback","title":"ExRocketmq.OffsetStore.get_offset/2","doc":"","ref":"ExRocketmq.OffsetStore.html#c:get_offset/2"},{"type":"callback","title":"ExRocketmq.OffsetStore.persist_offset/2","doc":"","ref":"ExRocketmq.OffsetStore.html#c:persist_offset/2"},{"type":"callback","title":"ExRocketmq.OffsetStore.read/2","doc":"","ref":"ExRocketmq.OffsetStore.html#c:read/2"},{"type":"callback","title":"ExRocketmq.OffsetStore.remove/2","doc":"","ref":"ExRocketmq.OffsetStore.html#c:remove/2"},{"type":"callback","title":"ExRocketmq.OffsetStore.update/3","doc":"","ref":"ExRocketmq.OffsetStore.html#c:update/3"},{"type":"type","title":"ExRocketmq.OffsetStore.t/0","doc":"","ref":"ExRocketmq.OffsetStore.html#t:t/0"},{"type":"module","title":"ExRocketmq.ProcessQueue.Concurrent","doc":"This module provides functions for concurrent message processing.\n\nThis module contains the `run/1` function, which is used to process messages concurrently.","ref":"ExRocketmq.ProcessQueue.Concurrent.html"},{"type":"function","title":"ExRocketmq.ProcessQueue.Concurrent.run/1","doc":"","ref":"ExRocketmq.ProcessQueue.Concurrent.html#run/1"},{"type":"module","title":"ExRocketmq.Producer","doc":"This module provides a GenServer implementation of a RocketMQ producer.","ref":"ExRocketmq.Producer.html"},{"type":"module","title":"Example - ExRocketmq.Producer","doc":"- First, write a producer task:\n```Elixir\ndefmodule DemoProducer do\n\n  use Task\n\n  alias ExRocketmq.Producer\n  alias ExRocketmq.Models.Message\n\n  @topic \"POETRY\"\n  @msgs \"风劲角弓鸣，将军猎渭城。\n  草枯鹰眼疾，雪尽马蹄轻。\n  忽过新丰市，还归细柳营。\n  回看射雕处，千里暮云平。\"\n\n  def start_link(opts) do\n    Task.start_link(__MODULE__, :run, [opts])\n  end\n\n  defp get_msg() do\n    @msgs\n    |> String.split(\"\n\")\n  end\n\n  def run(opts) do\n    get_msg()\n    |> Enum.each(fn msgs ->\n      to_emit =\n        msgs\n        |> Enum.map(fn msg ->\n          %Message{topic: @topic, body: msg}\n        end)\n\n      Producer.send_sync(:producer, to_emit)\n    end)\n\n    Process.sleep(5000)\n    run(opts)\n  end\nend\n```\n\n- Second, add namesrvs and producer and your msg produce task to supervisor tree:\n```Elixir\nchildren = [\n  {Namesrvs,\n   remotes: [\n     [transport: Transport.Tcp.new(host: \"test.rocket-mq.net\", port: 31_120)]\n   ],\n   opts: [\n     name: :namesrvs\n   ]},\n  {\n    Producer,\n    group_name: \"GID_POETRY\",\n    namesrvs: :namesrvs,\n    trace_enable: true,\n    opts: [\n      name: :producer\n    ]\n  },\n  {\n    DemoProducer,\n    []\n  }\n]\n\nSupervisor.start_link(children, strategy: :one_for_one)\n```\n\nThis module support both normal producer and transactional producer, if you want to use transactional producer,\nyou must implement `ExRocketmq.Producer.Transaction` behaviour, see this [example](https://github.com/tt67wq/ex_rocketmq/blob/master/examples/trans_producer.exs) for more details.","ref":"ExRocketmq.Producer.html#module-example"},{"type":"function","title":"ExRocketmq.Producer.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExRocketmq.Producer.html#child_spec/1"},{"type":"function","title":"ExRocketmq.Producer.handle_continue/2","doc":"","ref":"ExRocketmq.Producer.html#handle_continue/2"},{"type":"function","title":"ExRocketmq.Producer.init/1","doc":"","ref":"ExRocketmq.Producer.html#init/1"},{"type":"function","title":"ExRocketmq.Producer.send_oneway/2","doc":"send msgs to broker, and don't wait broker response","ref":"ExRocketmq.Producer.html#send_oneway/2"},{"type":"function","title":"Examples - ExRocketmq.Producer.send_oneway/2","doc":"iex> Producer.send_oneway(producer, [%Message{topic: topic, body: \"Hello from elixir\"}])\n    :ok","ref":"ExRocketmq.Producer.html#send_oneway/2-examples"},{"type":"function","title":"ExRocketmq.Producer.send_sync/2","doc":"send msgs to broker, and wait broker response","ref":"ExRocketmq.Producer.html#send_sync/2"},{"type":"function","title":"Examples - ExRocketmq.Producer.send_sync/2","doc":"iex> Producer.send_sync(producer, [%Message{topic: topic, body: \"Hello from elixir\"}])\n    {:ok,\n     %ExRocketmq.Models.SendMsg.Response{\n       status: 0,\n       msg_id: \"0A5804A1000051AF000000450648EF0E\",\n       queue: %ExRocketmq.Models.MessageQueue{\n         topic: \"SOME_TOPIC\",\n         broker_name: \"broker-0\",\n         queue_id: 0\n       },\n       queue_offset: 689717,\n       transaction_id: \"\",\n       offset_msg_id: \"0A5804A1000051AF000000450648EF0E\",\n       region_id: \"DefaultRegion\",\n       trace_on: true\n     }}","ref":"ExRocketmq.Producer.html#send_sync/2-examples"},{"type":"function","title":"ExRocketmq.Producer.send_transaction_msg/2","doc":"Sends a message to the broker as part of a transaction and waits for the broker's response.","ref":"ExRocketmq.Producer.html#send_transaction_msg/2"},{"type":"function","title":"Examples - ExRocketmq.Producer.send_transaction_msg/2","doc":"iex> Producer.send_transaction_msg(producer, %Message{topic: topic, body: \"Hello from elixir\"})\n    {:ok, response, transaction_state}","ref":"ExRocketmq.Producer.html#send_transaction_msg/2-examples"},{"type":"function","title":"ExRocketmq.Producer.start_link/1","doc":"start producer process","ref":"ExRocketmq.Producer.html#start_link/1"},{"type":"function","title":"Options - ExRocketmq.Producer.start_link/1","doc":"* `:group_name` (`t:String.t/0`) - Required. The group name of the producer\n\n* `:namesrvs` - Required. The namesrvs process\n\n* `:namespace` (`t:String.t/0`) - The namespace of the producer The default value is `\"\"`.\n\n* `:selector` (`t:term/0`) - The mq selector of the producer The default value is `%ExRocketmq.Producer.MqSelector.Random{}`.\n\n* `:compress` (`t:keyword/0`) - The compress opts of the producer\n\n  * `:compressor` (`t:atom/0`) - The compress implemention of the producer The default value is `ExRocketmq.Compress.Zlib`.\n\n  * `:compress_over` (`t:integer/0`) - Do compress when message bytesize over this value The default value is `4096`.\n\n  * `:compress_level` - The compress level of the compressor The default value is `:best_compression`.\n\n* `:transaction_listener` (`t:term/0`) - The transaction listener of the producer, must implement `ExRocketmq.Producer.Transaction`, see `ExRocketmq.Producer.Transaction` for more details\n\n* `:trace_enable` (`t:boolean/0`) - The trace enable of the producer The default value is `false`.\n\n* `:opts` (`t:keyword/0`) - The opts of the producer's GenServer The default value is `[]`.","ref":"ExRocketmq.Producer.html#start_link/1-options"},{"type":"function","title":"Examples - ExRocketmq.Producer.start_link/1","doc":"iex> ExRocketmq.Producer.start_link(\n      group_name: \"GID_POETRY\",\n      namesrvs: :namesrvs,\n      trace_enable: true,\n      opts: [\n        name: :producer\n      ]\n    )","ref":"ExRocketmq.Producer.html#start_link/1-examples"},{"type":"function","title":"ExRocketmq.Producer.stop/1","doc":"","ref":"ExRocketmq.Producer.html#stop/1"},{"type":"type","title":"ExRocketmq.Producer.producer_opts_schema_t/0","doc":"","ref":"ExRocketmq.Producer.html#t:producer_opts_schema_t/0"},{"type":"type","title":"ExRocketmq.Producer.publish_map/0","doc":"","ref":"ExRocketmq.Producer.html#t:publish_map/0"},{"type":"behaviour","title":"ExRocketmq.Producer.MqSelector","doc":"Provides a behaviour for selecting MessageQueues in the ExRocketmq Producer.\n\nMQ selectors implementing this behaviour are responsible for selecting the\nappropriate MessageQueue when sending messages to RocketMQ. This allows plugging\nin different selection strategies like random, round-robin, etc.\n\nTo use a custom MQ selector, implement the `new/1`, `start/1`, `stop/1` and\n`select/3` callbacks and pass the module as the `:mq_selector` option to the\n`ExRocketmq.Producer` constructor.","ref":"ExRocketmq.Producer.MqSelector.html"},{"type":"callback","title":"ExRocketmq.Producer.MqSelector.new/1","doc":"","ref":"ExRocketmq.Producer.MqSelector.html#c:new/1"},{"type":"function","title":"ExRocketmq.Producer.MqSelector.select/3","doc":"","ref":"ExRocketmq.Producer.MqSelector.html#select/3"},{"type":"callback","title":"ExRocketmq.Producer.MqSelector.select/3","doc":"","ref":"ExRocketmq.Producer.MqSelector.html#c:select/3"},{"type":"function","title":"ExRocketmq.Producer.MqSelector.start/1","doc":"","ref":"ExRocketmq.Producer.MqSelector.html#start/1"},{"type":"callback","title":"ExRocketmq.Producer.MqSelector.start/1","doc":"","ref":"ExRocketmq.Producer.MqSelector.html#c:start/1"},{"type":"function","title":"ExRocketmq.Producer.MqSelector.stop/1","doc":"","ref":"ExRocketmq.Producer.MqSelector.html#stop/1"},{"type":"callback","title":"ExRocketmq.Producer.MqSelector.stop/1","doc":"","ref":"ExRocketmq.Producer.MqSelector.html#c:stop/1"},{"type":"type","title":"ExRocketmq.Producer.MqSelector.t/0","doc":"","ref":"ExRocketmq.Producer.MqSelector.html#t:t/0"},{"type":"module","title":"ExRocketmq.Producer.MqSelector.Random","doc":"Implements random MessageQueue selection for the ExRocketmq Producer.\n\nThis module provides a random selection strategy by randomly choosing a\nMessageQueue from the given list on each `select/3` call.\n\nTo use this random selector, pass the `ExRocketmq.Producer.MqSelector.Random`\nmodule as the `:selector` option when constructing a Producer:\n\n  producer = ExRocketmq.Producer.start_link(\n    selector: ExRocketmq.Producer.MqSelector.Random.new([])\n  )","ref":"ExRocketmq.Producer.MqSelector.Random.html"},{"type":"function","title":"ExRocketmq.Producer.MqSelector.Random.new/1","doc":"","ref":"ExRocketmq.Producer.MqSelector.Random.html#new/1"},{"type":"function","title":"ExRocketmq.Producer.MqSelector.Random.select/3","doc":"","ref":"ExRocketmq.Producer.MqSelector.Random.html#select/3"},{"type":"function","title":"ExRocketmq.Producer.MqSelector.Random.start/1","doc":"","ref":"ExRocketmq.Producer.MqSelector.Random.html#start/1"},{"type":"function","title":"ExRocketmq.Producer.MqSelector.Random.stop/1","doc":"","ref":"ExRocketmq.Producer.MqSelector.Random.html#stop/1"},{"type":"type","title":"ExRocketmq.Producer.MqSelector.Random.t/0","doc":"","ref":"ExRocketmq.Producer.MqSelector.Random.html#t:t/0"},{"type":"module","title":"ExRocketmq.Producer.Selector.Hash","doc":"Implements hash-based MessageQueue selection for the ExRocketmq Producer.\n\nIt uses consistent hashing to map messages to queues based on the value of a\nsharding key property on the message. If no sharding key is present, it falls\nback to random selection.\n\nThe sharding key used is the `:sharding_key` property defined in the RocketMQ\nprotocol.\n\nTo use this hash-based selector, pass the\n`ExRocketmq.Producer.Selector.Hash` module as the `:selector` option when\nconstructing a Producer:\n\n  producer = ExRocketmq.Producer.start_link(\n    selector: ExRocketmq.Producer.Selector.Hash.new([])\n  )","ref":"ExRocketmq.Producer.Selector.Hash.html"},{"type":"function","title":"ExRocketmq.Producer.Selector.Hash.new/1","doc":"","ref":"ExRocketmq.Producer.Selector.Hash.html#new/1"},{"type":"function","title":"ExRocketmq.Producer.Selector.Hash.select/3","doc":"","ref":"ExRocketmq.Producer.Selector.Hash.html#select/3"},{"type":"function","title":"ExRocketmq.Producer.Selector.Hash.start/1","doc":"","ref":"ExRocketmq.Producer.Selector.Hash.html#start/1"},{"type":"function","title":"ExRocketmq.Producer.Selector.Hash.stop/1","doc":"","ref":"ExRocketmq.Producer.Selector.Hash.html#stop/1"},{"type":"type","title":"ExRocketmq.Producer.Selector.Hash.t/0","doc":"","ref":"ExRocketmq.Producer.Selector.Hash.html#t:t/0"},{"type":"module","title":"ExRocketmq.Producer.Selector.RoundRobin","doc":"Implements round-robin MessageQueue selection for the ExRocketmq Producer.\n\nIt maintains an internal counter via an Agent to track the next queue index. On\neach `select/3` call, it returns the queue at the current index and increments\nthe counter.\n\nTo use this round-robin selector, pass the\n`ExRocketmq.Producer.Selector.RoundRobin` module as the `:selector` option\nwhen constructing a Producer:\n\n  producer = ExRocketmq.Producer.start_link(\n    selector: ExRocketmq.Producer.Selector.RoundRobin.new([])\n  )","ref":"ExRocketmq.Producer.Selector.RoundRobin.html"},{"type":"function","title":"ExRocketmq.Producer.Selector.RoundRobin.new/1","doc":"","ref":"ExRocketmq.Producer.Selector.RoundRobin.html#new/1"},{"type":"function","title":"ExRocketmq.Producer.Selector.RoundRobin.select/3","doc":"","ref":"ExRocketmq.Producer.Selector.RoundRobin.html#select/3"},{"type":"function","title":"ExRocketmq.Producer.Selector.RoundRobin.start/1","doc":"","ref":"ExRocketmq.Producer.Selector.RoundRobin.html#start/1"},{"type":"function","title":"ExRocketmq.Producer.Selector.RoundRobin.stop/1","doc":"","ref":"ExRocketmq.Producer.Selector.RoundRobin.html#stop/1"},{"type":"type","title":"ExRocketmq.Producer.Selector.RoundRobin.t/0","doc":"","ref":"ExRocketmq.Producer.Selector.RoundRobin.html#t:t/0"},{"type":"behaviour","title":"ExRocketmq.Producer.Transaction","doc":"The implementation of transactional messages in RocketMQ is based on a three-step process:\n\n- Producers send a prepare message:\n  First, producers send a prepare message to the broker.\n  This message gets persisted in the commit log but is not pushed to consumers.\n  This ensures the message will not be consumed before the transaction is committed.\n\n- Producers execute transaction logic:\n  Producers call their own transaction execution logic, such as database operations.\n  This step is asynchronous and does not depend on RocketMQ.\n\n- Producers send a commit or rollback message:\n  If the transaction executes successfully, producers send a commit message to the broker.\n  The broker will then push the previously persisted prepare message to consumers, indicating it can now be consumed.\n  If the transaction fails, producers send a rollback message. The broker will delete the previously persisted prepare message,\n  and consumers will not receive that message.\n\nThrough these three steps, it can be ensured that messages will only be consumed if the transaction commits successfully,\nthus achieving atomicity.\n\nThis module provides a behaviour for implementing local transaction and check operations for RocketMQ producers.\nThe `execute_local` callback is called before a message is committed to the broker.\nIt accepts a `message` as an argument and should return `{:ok, state}` if the transaction was successful, or an error tuple otherwise.\n\nThe `check_local` callback is called after a message is committed to the broker.\nIt accepts a `message_ext` as an argument and should return `{:ok, state}` if the transaction was successful, or an error tuple otherwise.","ref":"ExRocketmq.Producer.Transaction.html"},{"type":"behaviour","title":"Examples - ExRocketmq.Producer.Transaction","doc":"Here's an example of how to implement the `ExRocketmq.Producer.Transaction` behaviour:\n\n    ```Elixir\n    defmodule MyTransaction do\n      @behaviour ExRocketmq.Producer.Transaction\n\n      @impl ExRocketmq.Producer.Transaction\n      def execute_local(_state, message) do\n        # perform local transaction\n        {:ok, :committed}\n      end\n\n      @impl ExRocketmq.Producer.Transaction\n      def check_local(_state, message_ext) do\n        # check local transaction status\n        {:ok, :committed}\n      end\n    end\n    ```","ref":"ExRocketmq.Producer.Transaction.html#module-examples"},{"type":"function","title":"ExRocketmq.Producer.Transaction.check_local/2","doc":"check if local transaction is ok or not","ref":"ExRocketmq.Producer.Transaction.html#check_local/2"},{"type":"callback","title":"ExRocketmq.Producer.Transaction.check_local/2","doc":"","ref":"ExRocketmq.Producer.Transaction.html#c:check_local/2"},{"type":"function","title":"ExRocketmq.Producer.Transaction.execute_local/2","doc":"execute local transaction before commit to broker","ref":"ExRocketmq.Producer.Transaction.html#execute_local/2"},{"type":"callback","title":"ExRocketmq.Producer.Transaction.execute_local/2","doc":"","ref":"ExRocketmq.Producer.Transaction.html#c:execute_local/2"},{"type":"type","title":"ExRocketmq.Producer.Transaction.t/0","doc":"","ref":"ExRocketmq.Producer.Transaction.html#t:t/0"},{"type":"module","title":"ExRocketmq.Protocol.ConsumeReturnType","doc":"Consume return type","ref":"ExRocketmq.Protocol.ConsumeReturnType.html"},{"type":"macro","title":"ExRocketmq.Protocol.ConsumeReturnType.exception/0","doc":"","ref":"ExRocketmq.Protocol.ConsumeReturnType.html#exception/0"},{"type":"macro","title":"ExRocketmq.Protocol.ConsumeReturnType.failed/0","doc":"","ref":"ExRocketmq.Protocol.ConsumeReturnType.html#failed/0"},{"type":"macro","title":"ExRocketmq.Protocol.ConsumeReturnType.null/0","doc":"","ref":"ExRocketmq.Protocol.ConsumeReturnType.html#null/0"},{"type":"macro","title":"ExRocketmq.Protocol.ConsumeReturnType.success/0","doc":"","ref":"ExRocketmq.Protocol.ConsumeReturnType.html#success/0"},{"type":"macro","title":"ExRocketmq.Protocol.ConsumeReturnType.timeout/0","doc":"","ref":"ExRocketmq.Protocol.ConsumeReturnType.html#timeout/0"},{"type":"module","title":"ExRocketmq.Protocol.Flag","doc":"sysflag constants","ref":"ExRocketmq.Protocol.Flag.html"},{"type":"macro","title":"ExRocketmq.Protocol.Flag.flag_born_host_v6/0","doc":"","ref":"ExRocketmq.Protocol.Flag.html#flag_born_host_v6/0"},{"type":"macro","title":"ExRocketmq.Protocol.Flag.flag_compressed/0","doc":"","ref":"ExRocketmq.Protocol.Flag.html#flag_compressed/0"},{"type":"macro","title":"ExRocketmq.Protocol.Flag.flag_transaction_prepared/0","doc":"","ref":"ExRocketmq.Protocol.Flag.html#flag_transaction_prepared/0"},{"type":"module","title":"ExRocketmq.Protocol.Properties","doc":"RocketMQ properties","ref":"ExRocketmq.Protocol.Properties.html"},{"type":"macro","title":"ExRocketmq.Protocol.Properties.property_msg_type/0","doc":"","ref":"ExRocketmq.Protocol.Properties.html#property_msg_type/0"},{"type":"macro","title":"ExRocketmq.Protocol.Properties.property_producer_group/0","doc":"","ref":"ExRocketmq.Protocol.Properties.html#property_producer_group/0"},{"type":"macro","title":"ExRocketmq.Protocol.Properties.property_sharding_key/0","doc":"","ref":"ExRocketmq.Protocol.Properties.html#property_sharding_key/0"},{"type":"macro","title":"ExRocketmq.Protocol.Properties.property_transaction_id/0","doc":"","ref":"ExRocketmq.Protocol.Properties.html#property_transaction_id/0"},{"type":"macro","title":"ExRocketmq.Protocol.Properties.property_transaction_prepared/0","doc":"","ref":"ExRocketmq.Protocol.Properties.html#property_transaction_prepared/0"},{"type":"macro","title":"ExRocketmq.Protocol.Properties.property_unique_client_msgid_key/0","doc":"","ref":"ExRocketmq.Protocol.Properties.html#property_unique_client_msgid_key/0"},{"type":"module","title":"ExRocketmq.Protocol.PullStatus","doc":"The pull status constants","ref":"ExRocketmq.Protocol.PullStatus.html"},{"type":"macro","title":"ExRocketmq.Protocol.PullStatus.pull_broker_timeout/0","doc":"","ref":"ExRocketmq.Protocol.PullStatus.html#pull_broker_timeout/0"},{"type":"macro","title":"ExRocketmq.Protocol.PullStatus.pull_found/0","doc":"","ref":"ExRocketmq.Protocol.PullStatus.html#pull_found/0"},{"type":"macro","title":"ExRocketmq.Protocol.PullStatus.pull_no_matched_msg/0","doc":"","ref":"ExRocketmq.Protocol.PullStatus.html#pull_no_matched_msg/0"},{"type":"macro","title":"ExRocketmq.Protocol.PullStatus.pull_no_new_msg/0","doc":"","ref":"ExRocketmq.Protocol.PullStatus.html#pull_no_new_msg/0"},{"type":"macro","title":"ExRocketmq.Protocol.PullStatus.pull_offset_illegal/0","doc":"","ref":"ExRocketmq.Protocol.PullStatus.html#pull_offset_illegal/0"},{"type":"module","title":"ExRocketmq.Protocol.Request","doc":"The request code constants","ref":"ExRocketmq.Protocol.Request.html"},{"type":"macro","title":"ExRocketmq.Protocol.Request.req_check_transaction_state/0","doc":"","ref":"ExRocketmq.Protocol.Request.html#req_check_transaction_state/0"},{"type":"macro","title":"ExRocketmq.Protocol.Request.req_consume_message_directly/0","doc":"","ref":"ExRocketmq.Protocol.Request.html#req_consume_message_directly/0"},{"type":"macro","title":"ExRocketmq.Protocol.Request.req_consumer_send_msg_back/0","doc":"","ref":"ExRocketmq.Protocol.Request.html#req_consumer_send_msg_back/0"},{"type":"macro","title":"ExRocketmq.Protocol.Request.req_end_transaction/0","doc":"","ref":"ExRocketmq.Protocol.Request.html#req_end_transaction/0"},{"type":"macro","title":"ExRocketmq.Protocol.Request.req_get_broker_cluster_info/0","doc":"","ref":"ExRocketmq.Protocol.Request.html#req_get_broker_cluster_info/0"},{"type":"macro","title":"ExRocketmq.Protocol.Request.req_get_consumer_list_by_group/0","doc":"","ref":"ExRocketmq.Protocol.Request.html#req_get_consumer_list_by_group/0"},{"type":"macro","title":"ExRocketmq.Protocol.Request.req_get_consumer_running_info/0","doc":"","ref":"ExRocketmq.Protocol.Request.html#req_get_consumer_running_info/0"},{"type":"macro","title":"ExRocketmq.Protocol.Request.req_get_max_offset/0","doc":"","ref":"ExRocketmq.Protocol.Request.html#req_get_max_offset/0"},{"type":"macro","title":"ExRocketmq.Protocol.Request.req_get_routeinfo_by_topic/0","doc":"","ref":"ExRocketmq.Protocol.Request.html#req_get_routeinfo_by_topic/0"},{"type":"macro","title":"ExRocketmq.Protocol.Request.req_heartbeat/0","doc":"","ref":"ExRocketmq.Protocol.Request.html#req_heartbeat/0"},{"type":"macro","title":"ExRocketmq.Protocol.Request.req_lock_batch_mq/0","doc":"","ref":"ExRocketmq.Protocol.Request.html#req_lock_batch_mq/0"},{"type":"macro","title":"ExRocketmq.Protocol.Request.req_notify_consumer_ids_changed/0","doc":"","ref":"ExRocketmq.Protocol.Request.html#req_notify_consumer_ids_changed/0"},{"type":"macro","title":"ExRocketmq.Protocol.Request.req_pull_message/0","doc":"","ref":"ExRocketmq.Protocol.Request.html#req_pull_message/0"},{"type":"macro","title":"ExRocketmq.Protocol.Request.req_query_consumer_offset/0","doc":"","ref":"ExRocketmq.Protocol.Request.html#req_query_consumer_offset/0"},{"type":"macro","title":"ExRocketmq.Protocol.Request.req_search_offset_by_timestamp/0","doc":"","ref":"ExRocketmq.Protocol.Request.html#req_search_offset_by_timestamp/0"},{"type":"macro","title":"ExRocketmq.Protocol.Request.req_send_batch_message/0","doc":"","ref":"ExRocketmq.Protocol.Request.html#req_send_batch_message/0"},{"type":"macro","title":"ExRocketmq.Protocol.Request.req_send_message/0","doc":"","ref":"ExRocketmq.Protocol.Request.html#req_send_message/0"},{"type":"macro","title":"ExRocketmq.Protocol.Request.req_send_reply_message/0","doc":"","ref":"ExRocketmq.Protocol.Request.html#req_send_reply_message/0"},{"type":"macro","title":"ExRocketmq.Protocol.Request.req_unlock_batch_mq/0","doc":"","ref":"ExRocketmq.Protocol.Request.html#req_unlock_batch_mq/0"},{"type":"macro","title":"ExRocketmq.Protocol.Request.req_update_consumer_offset/0","doc":"","ref":"ExRocketmq.Protocol.Request.html#req_update_consumer_offset/0"},{"type":"module","title":"ExRocketmq.Protocol.Response","doc":"The response code constants","ref":"ExRocketmq.Protocol.Response.html"},{"type":"macro","title":"ExRocketmq.Protocol.Response.res_flush_disk_timeout/0","doc":"","ref":"ExRocketmq.Protocol.Response.html#res_flush_disk_timeout/0"},{"type":"macro","title":"ExRocketmq.Protocol.Response.res_flush_slave_timeout/0","doc":"","ref":"ExRocketmq.Protocol.Response.html#res_flush_slave_timeout/0"},{"type":"macro","title":"ExRocketmq.Protocol.Response.res_no_permission/0","doc":"","ref":"ExRocketmq.Protocol.Response.html#res_no_permission/0"},{"type":"macro","title":"ExRocketmq.Protocol.Response.res_service_not_available/0","doc":"","ref":"ExRocketmq.Protocol.Response.html#res_service_not_available/0"},{"type":"macro","title":"ExRocketmq.Protocol.Response.res_slave_not_available/0","doc":"","ref":"ExRocketmq.Protocol.Response.html#res_slave_not_available/0"},{"type":"macro","title":"ExRocketmq.Protocol.Response.resp_error/0","doc":"","ref":"ExRocketmq.Protocol.Response.html#resp_error/0"},{"type":"macro","title":"ExRocketmq.Protocol.Response.resp_pull_not_found/0","doc":"","ref":"ExRocketmq.Protocol.Response.html#resp_pull_not_found/0"},{"type":"macro","title":"ExRocketmq.Protocol.Response.resp_pull_offset_moved/0","doc":"","ref":"ExRocketmq.Protocol.Response.html#resp_pull_offset_moved/0"},{"type":"macro","title":"ExRocketmq.Protocol.Response.resp_pull_retry_immediately/0","doc":"","ref":"ExRocketmq.Protocol.Response.html#resp_pull_retry_immediately/0"},{"type":"macro","title":"ExRocketmq.Protocol.Response.resp_success/0","doc":"","ref":"ExRocketmq.Protocol.Response.html#resp_success/0"},{"type":"macro","title":"ExRocketmq.Protocol.Response.resp_topic_not_exist/0","doc":"","ref":"ExRocketmq.Protocol.Response.html#resp_topic_not_exist/0"},{"type":"module","title":"ExRocketmq.Protocol.SendStatus","doc":"The send status constants","ref":"ExRocketmq.Protocol.SendStatus.html"},{"type":"macro","title":"ExRocketmq.Protocol.SendStatus.send_flush_disk_timeout/0","doc":"","ref":"ExRocketmq.Protocol.SendStatus.html#send_flush_disk_timeout/0"},{"type":"macro","title":"ExRocketmq.Protocol.SendStatus.send_flush_slave_timeout/0","doc":"","ref":"ExRocketmq.Protocol.SendStatus.html#send_flush_slave_timeout/0"},{"type":"macro","title":"ExRocketmq.Protocol.SendStatus.send_ok/0","doc":"","ref":"ExRocketmq.Protocol.SendStatus.html#send_ok/0"},{"type":"macro","title":"ExRocketmq.Protocol.SendStatus.send_slave_not_available/0","doc":"","ref":"ExRocketmq.Protocol.SendStatus.html#send_slave_not_available/0"},{"type":"macro","title":"ExRocketmq.Protocol.SendStatus.send_unknown_error/0","doc":"","ref":"ExRocketmq.Protocol.SendStatus.html#send_unknown_error/0"},{"type":"module","title":"ExRocketmq.Puller.Locked","doc":"Similar to the normal puller, but it will first try to acquire a lock on the queue","ref":"ExRocketmq.Puller.Locked.html"},{"type":"function","title":"ExRocketmq.Puller.Locked.run/1","doc":"","ref":"ExRocketmq.Puller.Locked.html#run/1"},{"type":"module","title":"ExRocketmq.Puller.Normal","doc":"This module contains the implementation of the normal puller for RocketMQ.\nIt handles pulling messages from the broker and processing them.\n\nThe `run/1` function is the entry point for the puller. It retrieves the next offset\nand continues running.\n\nNote: This module assumes the presence of other modules and structs from the\n`ExRocketmq` and `ExRocketmq.Models` namespaces.\n\nFor more information, refer to the RocketMQ documentation.","ref":"ExRocketmq.Puller.Normal.html"},{"type":"function","title":"ExRocketmq.Puller.Normal.run/1","doc":"","ref":"ExRocketmq.Puller.Normal.html#run/1"},{"type":"module","title":"ExRocketmq.Remote","doc":"The remote layer of the rocketmq: how client communicates with the nameserver or broker.\nRemote support 3 functions:\n- rpc: request-reply\n- one_way: send msg and don't wait for the response\n- pop_notify: pop the notify msg from the queue","ref":"ExRocketmq.Remote.html"},{"type":"function","title":"ExRocketmq.Remote.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExRocketmq.Remote.html#child_spec/1"},{"type":"function","title":"ExRocketmq.Remote.handle_continue/2","doc":"","ref":"ExRocketmq.Remote.html#handle_continue/2"},{"type":"function","title":"ExRocketmq.Remote.init/1","doc":"","ref":"ExRocketmq.Remote.html#init/1"},{"type":"function","title":"ExRocketmq.Remote.one_way/2","doc":"send msg to the remote server, and don't wait for the response","ref":"ExRocketmq.Remote.html#one_way/2"},{"type":"function","title":"Examples - ExRocketmq.Remote.one_way/2","doc":"iex> :ok = ExRocketmq.Remote.one_way(remote, msg)","ref":"ExRocketmq.Remote.html#one_way/2-examples"},{"type":"function","title":"ExRocketmq.Remote.pop_notify/1","doc":"","ref":"ExRocketmq.Remote.html#pop_notify/1"},{"type":"function","title":"ExRocketmq.Remote.rpc/3","doc":"make a rpc call to the remote server, and return the response","ref":"ExRocketmq.Remote.html#rpc/3"},{"type":"function","title":"Examples - ExRocketmq.Remote.rpc/3","doc":"iex> {:ok, _res} = ExRocketmq.Remote.rpc(remote, msg)","ref":"ExRocketmq.Remote.html#rpc/3-examples"},{"type":"function","title":"ExRocketmq.Remote.start_link/1","doc":"","ref":"ExRocketmq.Remote.html#start_link/1"},{"type":"function","title":"ExRocketmq.Remote.stop/1","doc":"stop remote server","ref":"ExRocketmq.Remote.html#stop/1"},{"type":"function","title":"ExRocketmq.Remote.transport_info/1","doc":"get transport running info","ref":"ExRocketmq.Remote.html#transport_info/1"},{"type":"function","title":"Examples - ExRocketmq.Remote.transport_info/1","doc":"iex> ExRocketmq.Remote.transport_info(remote)\n    {:ok, %{host: \"some host\", port: 1234, pid: #PID }}","ref":"ExRocketmq.Remote.html#transport_info/1-examples"},{"type":"type","title":"ExRocketmq.Remote.remote_opts_schema_t/0","doc":"","ref":"ExRocketmq.Remote.html#t:remote_opts_schema_t/0"},{"type":"behaviour","title":"ExRocketmq.Remote.ExtFields","doc":"behavior of ext_fields","ref":"ExRocketmq.Remote.ExtFields.html"},{"type":"function","title":"ExRocketmq.Remote.ExtFields.to_map/1","doc":"convert a struct to a jsonable string->string dict","ref":"ExRocketmq.Remote.ExtFields.html#to_map/1"},{"type":"callback","title":"ExRocketmq.Remote.ExtFields.to_map/1","doc":"","ref":"ExRocketmq.Remote.ExtFields.html#c:to_map/1"},{"type":"type","title":"ExRocketmq.Remote.ExtFields.t/0","doc":"","ref":"ExRocketmq.Remote.ExtFields.html#t:t/0"},{"type":"module","title":"ExRocketmq.Remote.Packet","doc":"The communication package of RocketMQ is divided into four parts:\n\n```text\n|-- Length --|-- Header Length --|-- Header --|-- Body --|\n```\n\n1. Header: This is the header of the protocol, and the data is serialized in JSON format.\n  Each key field in the JSON is fixed, but the fields for different communication requests may vary.\n2. Body: This contains the actual binary data of the request.\n  For example, in a network request to send a message, the body would contain the actual message content.\n3. Length: This represents the overall length of the package, which is a four-byte integer.\n4. Header length: This indicates the length of the header, also represented as a four-byte integer.","ref":"ExRocketmq.Remote.Packet.html"},{"type":"module","title":"Header explanation: - ExRocketmq.Remote.Packet","doc":"- code: Request/Response code. see `ExRocketmq.Protocol.Request` and `ExRocketmq.Protocol.Response` for details.\n\n- language: Since support for multiple languages is required, this field can inform both\n  communicating parties about the programming language used in the communication layer.\n\n- version: Informs the communication layer about the version number of the other party.\n  The responding party can use this information to perform special operations for compatibility with older versions,\n  among other things.\n\n- opaque: Request identification code. this is simply an incrementing integer used to match the corresponding request with its response.\n\n- flag: Bit interpretation.\n    The 0th bit indicates whether the communication is a request or a response. 0 represents a request, while 1 represents a response.\n    The 1st bit indicates whether the request is a one-way request. 1 represents a one-way request.\n    When handling a one-way request, the responding party does not provide a response,\n    and the requesting party does not need to wait for a response from the responding party.\n\n- remark: Additional text information. Commonly used to store exception messages returned by brokers/nameservers,\n    facilitating problem localization for developers.\n\n- extFields: This field is a string-dict, and it differs for each request/response and is completely customizable.","ref":"ExRocketmq.Remote.Packet.html#module-header-explanation"},{"type":"macro","title":"ExRocketmq.Remote.Packet.packet/1","doc":"","ref":"ExRocketmq.Remote.Packet.html#packet/1"},{"type":"macro","title":"ExRocketmq.Remote.Packet.packet/2","doc":"","ref":"ExRocketmq.Remote.Packet.html#packet/2"},{"type":"function","title":"ExRocketmq.Remote.Packet.response_type?/1","doc":"","ref":"ExRocketmq.Remote.Packet.html#response_type?/1"},{"type":"type","title":"ExRocketmq.Remote.Packet.t/0","doc":"","ref":"ExRocketmq.Remote.Packet.html#t:t/0"},{"type":"behaviour","title":"ExRocketmq.Remote.Serializer","doc":"encode/decode the packet to be sent or received","ref":"ExRocketmq.Remote.Serializer.html"},{"type":"function","title":"ExRocketmq.Remote.Serializer.decode/2","doc":"","ref":"ExRocketmq.Remote.Serializer.html#decode/2"},{"type":"callback","title":"ExRocketmq.Remote.Serializer.decode/2","doc":"","ref":"ExRocketmq.Remote.Serializer.html#c:decode/2"},{"type":"function","title":"ExRocketmq.Remote.Serializer.encode/2","doc":"","ref":"ExRocketmq.Remote.Serializer.html#encode/2"},{"type":"callback","title":"ExRocketmq.Remote.Serializer.encode/2","doc":"","ref":"ExRocketmq.Remote.Serializer.html#c:encode/2"},{"type":"callback","title":"ExRocketmq.Remote.Serializer.new/1","doc":"","ref":"ExRocketmq.Remote.Serializer.html#c:new/1"},{"type":"type","title":"ExRocketmq.Remote.Serializer.t/0","doc":"","ref":"ExRocketmq.Remote.Serializer.html#t:t/0"},{"type":"module","title":"ExRocketmq.Remote.Serializer.Json","doc":"encode/decode the packet to be sent or received\n\nFrame format:\n++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n| frame_size | header_length |         header_body        |     body     |\n++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n|   4bytes   |     4bytes    | (21 + r_len + e_len) bytes | remain bytes |\n++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++","ref":"ExRocketmq.Remote.Serializer.Json.html"},{"type":"function","title":"ExRocketmq.Remote.Serializer.Json.decode/2","doc":"","ref":"ExRocketmq.Remote.Serializer.Json.html#decode/2"},{"type":"function","title":"ExRocketmq.Remote.Serializer.Json.encode/2","doc":"","ref":"ExRocketmq.Remote.Serializer.Json.html#encode/2"},{"type":"function","title":"ExRocketmq.Remote.Serializer.Json.new/1","doc":"","ref":"ExRocketmq.Remote.Serializer.Json.html#new/1"},{"type":"type","title":"ExRocketmq.Remote.Serializer.Json.t/0","doc":"","ref":"ExRocketmq.Remote.Serializer.Json.html#t:t/0"},{"type":"module","title":"ExRocketmq.Remote.Waiter","doc":"The future-wait store of the remote, opaque => request pid\nThis module is a wrapper of ets and genserver, ets is responsible for storing the request,\nand GenServer is responsible for cleaning up the expired request","ref":"ExRocketmq.Remote.Waiter.html"},{"type":"function","title":"ExRocketmq.Remote.Waiter.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExRocketmq.Remote.Waiter.html#child_spec/1"},{"type":"function","title":"ExRocketmq.Remote.Waiter.pop/2","doc":"pop the value of the key from the waiter","ref":"ExRocketmq.Remote.Waiter.html#pop/2"},{"type":"function","title":"Examples - ExRocketmq.Remote.Waiter.pop/2","doc":"iex> ExRocketmq.Remote.Waiter.pop(waiter, 1)\n    nil","ref":"ExRocketmq.Remote.Waiter.html#pop/2-examples"},{"type":"function","title":"ExRocketmq.Remote.Waiter.put/4","doc":"put key-value pair to the waiter","ref":"ExRocketmq.Remote.Waiter.html#put/4"},{"type":"function","title":"Params - ExRocketmq.Remote.Waiter.put/4","doc":"- `waiter` - the waiter instance\n- `key` - the key of the value, must be an integer\n- `value` - the value of the key\n- `ttl` - the time to live(milliseconds) of the key-value pair","ref":"ExRocketmq.Remote.Waiter.html#put/4-params"},{"type":"function","title":"Examples - ExRocketmq.Remote.Waiter.put/4","doc":"iex> ExRocketmq.Remote.Waiter.put(waiter, 1, return_value, ttl: 5000)","ref":"ExRocketmq.Remote.Waiter.html#put/4-examples"},{"type":"function","title":"ExRocketmq.Remote.Waiter.start/1","doc":"start the waiter","ref":"ExRocketmq.Remote.Waiter.html#start/1"},{"type":"function","title":"Options - ExRocketmq.Remote.Waiter.start/1","doc":"* `:name` (`t:atom/0`) - Required. The name of the waiter\n\n* `:interval` (`t:integer/0`) - The cleanup interval time in millisecond of the waiter The default value is `5000`.\n\n* `:opts` (`t:keyword/0`) - The other options of the waiter The default value is `[]`.","ref":"ExRocketmq.Remote.Waiter.html#start/1-options"},{"type":"function","title":"Examples - ExRocketmq.Remote.Waiter.start/1","doc":"iex> ExRocketmq.Remote.Waiter.start(name: :waiter_name)\n    %ExRocketmq.Remote.Waiter{\n      name: :waiter_name,\n      pid: #PID ,\n      interval: 5000\n    }","ref":"ExRocketmq.Remote.Waiter.html#start/1-examples"},{"type":"function","title":"ExRocketmq.Remote.Waiter.stop/1","doc":"","ref":"ExRocketmq.Remote.Waiter.html#stop/1"},{"type":"type","title":"ExRocketmq.Remote.Waiter.key/0","doc":"","ref":"ExRocketmq.Remote.Waiter.html#t:key/0"},{"type":"type","title":"ExRocketmq.Remote.Waiter.opts_schema_t/0","doc":"","ref":"ExRocketmq.Remote.Waiter.html#t:opts_schema_t/0"},{"type":"type","title":"ExRocketmq.Remote.Waiter.t/0","doc":"","ref":"ExRocketmq.Remote.Waiter.html#t:t/0"},{"type":"type","title":"ExRocketmq.Remote.Waiter.value/0","doc":"","ref":"ExRocketmq.Remote.Waiter.html#t:value/0"},{"type":"module","title":"ExRocketmq.Stats","doc":"{mq, :pull} => {locked, last_lock_timestamp, last_pull_timestamp}\n{mq, :consume} => last_consumed_timestamp\n\npull_point: {ts, round, cost}\nconsume_point: {ts, round, ok_round, failed_round, cost, failed_cnt}\n\ntopic => {begin_pull_point, end_pull_point}\ntopic => {begin_consume_point, end_consume_point}","ref":"ExRocketmq.Stats.html"},{"type":"function","title":"ExRocketmq.Stats.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExRocketmq.Stats.html#child_spec/1"},{"type":"function","title":"ExRocketmq.Stats.consume_report/5","doc":"","ref":"ExRocketmq.Stats.html#consume_report/5"},{"type":"function","title":"ExRocketmq.Stats.get_running_info/1","doc":"","ref":"ExRocketmq.Stats.html#get_running_info/1"},{"type":"function","title":"ExRocketmq.Stats.init/1","doc":"","ref":"ExRocketmq.Stats.html#init/1"},{"type":"function","title":"ExRocketmq.Stats.puller_report/6","doc":"","ref":"ExRocketmq.Stats.html#puller_report/6"},{"type":"function","title":"ExRocketmq.Stats.start_link/1","doc":"","ref":"ExRocketmq.Stats.html#start_link/1"},{"type":"module","title":"ExRocketmq.Tracer","doc":"Trace data emiter","ref":"ExRocketmq.Tracer.html"},{"type":"function","title":"ExRocketmq.Tracer.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExRocketmq.Tracer.html#child_spec/1"},{"type":"function","title":"ExRocketmq.Tracer.handle_continue/2","doc":"","ref":"ExRocketmq.Tracer.html#handle_continue/2"},{"type":"function","title":"ExRocketmq.Tracer.init/1","doc":"","ref":"ExRocketmq.Tracer.html#init/1"},{"type":"function","title":"ExRocketmq.Tracer.send_trace/2","doc":"","ref":"ExRocketmq.Tracer.html#send_trace/2"},{"type":"function","title":"ExRocketmq.Tracer.start_link/1","doc":"","ref":"ExRocketmq.Tracer.html#start_link/1"},{"type":"function","title":"ExRocketmq.Tracer.stop/1","doc":"","ref":"ExRocketmq.Tracer.html#stop/1"},{"type":"type","title":"ExRocketmq.Tracer.trace_opts_schema_t/0","doc":"","ref":"ExRocketmq.Tracer.html#t:trace_opts_schema_t/0"},{"type":"behaviour","title":"ExRocketmq.Transport","doc":"The transport layer of the rocketmq protocol, default implementation is ExRocketmq.Transport.TCP","ref":"ExRocketmq.Transport.html"},{"type":"function","title":"ExRocketmq.Transport.info/1","doc":"get the info of transport","ref":"ExRocketmq.Transport.html#info/1"},{"type":"callback","title":"ExRocketmq.Transport.info/1","doc":"","ref":"ExRocketmq.Transport.html#c:info/1"},{"type":"callback","title":"ExRocketmq.Transport.new/1","doc":"","ref":"ExRocketmq.Transport.html#c:new/1"},{"type":"function","title":"ExRocketmq.Transport.output/2","doc":"output a pkt by transport layer","ref":"ExRocketmq.Transport.html#output/2"},{"type":"callback","title":"ExRocketmq.Transport.output/2","doc":"","ref":"ExRocketmq.Transport.html#c:output/2"},{"type":"function","title":"ExRocketmq.Transport.recv/1","doc":"recv a packet from transport layer","ref":"ExRocketmq.Transport.html#recv/1"},{"type":"callback","title":"ExRocketmq.Transport.recv/1","doc":"","ref":"ExRocketmq.Transport.html#c:recv/1"},{"type":"function","title":"ExRocketmq.Transport.start/1","doc":"start transport module","ref":"ExRocketmq.Transport.html#start/1"},{"type":"callback","title":"ExRocketmq.Transport.start/1","doc":"","ref":"ExRocketmq.Transport.html#c:start/1"},{"type":"function","title":"ExRocketmq.Transport.stop/1","doc":"stop transport module","ref":"ExRocketmq.Transport.html#stop/1"},{"type":"callback","title":"ExRocketmq.Transport.stop/1","doc":"","ref":"ExRocketmq.Transport.html#c:stop/1"},{"type":"type","title":"ExRocketmq.Transport.error_t/0","doc":"","ref":"ExRocketmq.Transport.html#t:error_t/0"},{"type":"type","title":"ExRocketmq.Transport.t/0","doc":"","ref":"ExRocketmq.Transport.html#t:t/0"},{"type":"module","title":"ExRocketmq.Transport.Tcp","doc":"Implement the transport layer of the rocketmq protocol via tcp","ref":"ExRocketmq.Transport.Tcp.html"},{"type":"function","title":"ExRocketmq.Transport.Tcp.info/1","doc":"return infomaion of the transport","ref":"ExRocketmq.Transport.Tcp.html#info/1"},{"type":"function","title":"Examples - ExRocketmq.Transport.Tcp.info/1","doc":"iex> info(transport)\n    {:ok, %{pid: pid, host: host, port: port}}","ref":"ExRocketmq.Transport.Tcp.html#info/1-examples"},{"type":"function","title":"ExRocketmq.Transport.Tcp.new/1","doc":"create new instance of the tcp transport","ref":"ExRocketmq.Transport.Tcp.html#new/1"},{"type":"function","title":"Options - ExRocketmq.Transport.Tcp.new/1","doc":"* `:host` (`t:String.t/0`) - Required. The host of the nameserver\n\n* `:port` (`t:integer/0`) - Required. The port of the nameserver\n\n* `:timeout` (`t:integer/0`) - The timeout of the transport The default value is `5000`.\n\n* `:sockopts` (list of `t:term/0`) - The socket options of the transport The default value is `[]`.\n\n* `:opts` (`t:keyword/0`) - The other options of the transport The default value is `[]`.","ref":"ExRocketmq.Transport.Tcp.html#new/1-options"},{"type":"function","title":"Examples - ExRocketmq.Transport.Tcp.new/1","doc":"iex> new(host: \"some host\", port: 1234)\n    %ExRocketmq.Transport.Tcp{\n      pid: nil,\n      host: \"some host\",\n      port: 1234,\n      timeout: 5000,\n      sockopts: [],\n      opts: []\n    }","ref":"ExRocketmq.Transport.Tcp.html#new/1-examples"},{"type":"function","title":"ExRocketmq.Transport.Tcp.output/2","doc":"output a binary data through the transport socket","ref":"ExRocketmq.Transport.Tcp.html#output/2"},{"type":"function","title":"Examples - ExRocketmq.Transport.Tcp.output/2","doc":"iex> output(transport, \"some data\")\n    :ok","ref":"ExRocketmq.Transport.Tcp.html#output/2-examples"},{"type":"function","title":"ExRocketmq.Transport.Tcp.recv/1","doc":"recv a binary packet from the transport socket","ref":"ExRocketmq.Transport.Tcp.html#recv/1"},{"type":"function","title":"Examples - ExRocketmq.Transport.Tcp.recv/1","doc":"iex> recv(transport)\n    {:ok, \"some data\"}","ref":"ExRocketmq.Transport.Tcp.html#recv/1-examples"},{"type":"function","title":"ExRocketmq.Transport.Tcp.start/1","doc":"`start` starts Genserver of the transport and set the `pid` field of the transport","ref":"ExRocketmq.Transport.Tcp.html#start/1"},{"type":"type","title":"ExRocketmq.Transport.Tcp.t/0","doc":"The type of the transport\n- pid: the pid of the transport, the `start` function will set this field;\n- host: the host of the target server;\n- port: the port of the target server;\n- timeout: connection timeout in milliseconds;\n- sockopts: the socket options of gen_tcp;\n- opts: the other options of the transport;","ref":"ExRocketmq.Transport.Tcp.html#t:t/0"},{"type":"type","title":"ExRocketmq.Transport.Tcp.tcp_opts_schema_t/0","doc":"","ref":"ExRocketmq.Transport.Tcp.html#t:tcp_opts_schema_t/0"},{"type":"module","title":"ExRocketmq.Typespecs","doc":"all kinds of typespecs","ref":"ExRocketmq.Typespecs.html"},{"type":"type","title":"ExRocketmq.Typespecs.broker_name/0","doc":"","ref":"ExRocketmq.Typespecs.html#t:broker_name/0"},{"type":"type","title":"ExRocketmq.Typespecs.compress_level/0","doc":"","ref":"ExRocketmq.Typespecs.html#t:compress_level/0"},{"type":"type","title":"ExRocketmq.Typespecs.consume_from_where/0","doc":"","ref":"ExRocketmq.Typespecs.html#t:consume_from_where/0"},{"type":"type","title":"ExRocketmq.Typespecs.consumer_model/0","doc":"","ref":"ExRocketmq.Typespecs.html#t:consumer_model/0"},{"type":"type","title":"ExRocketmq.Typespecs.delay_level/0","doc":"","ref":"ExRocketmq.Typespecs.html#t:delay_level/0"},{"type":"type","title":"ExRocketmq.Typespecs.error_t/0","doc":"","ref":"ExRocketmq.Typespecs.html#t:error_t/0"},{"type":"type","title":"ExRocketmq.Typespecs.ext_fields/0","doc":"","ref":"ExRocketmq.Typespecs.html#t:ext_fields/0"},{"type":"type","title":"ExRocketmq.Typespecs.flag/0","doc":"","ref":"ExRocketmq.Typespecs.html#t:flag/0"},{"type":"type","title":"ExRocketmq.Typespecs.group_name/0","doc":"","ref":"ExRocketmq.Typespecs.html#t:group_name/0"},{"type":"type","title":"ExRocketmq.Typespecs.msg_id/0","doc":"","ref":"ExRocketmq.Typespecs.html#t:msg_id/0"},{"type":"type","title":"ExRocketmq.Typespecs.name/0","doc":"","ref":"ExRocketmq.Typespecs.html#t:name/0"},{"type":"type","title":"ExRocketmq.Typespecs.namespace/0","doc":"","ref":"ExRocketmq.Typespecs.html#t:namespace/0"},{"type":"type","title":"ExRocketmq.Typespecs.ok_t/0","doc":"","ref":"ExRocketmq.Typespecs.html#t:ok_t/0"},{"type":"type","title":"ExRocketmq.Typespecs.ok_t/1","doc":"","ref":"ExRocketmq.Typespecs.html#t:ok_t/1"},{"type":"type","title":"ExRocketmq.Typespecs.on_start/0","doc":"","ref":"ExRocketmq.Typespecs.html#t:on_start/0"},{"type":"type","title":"ExRocketmq.Typespecs.opaque/0","doc":"","ref":"ExRocketmq.Typespecs.html#t:opaque/0"},{"type":"type","title":"ExRocketmq.Typespecs.opts/0","doc":"","ref":"ExRocketmq.Typespecs.html#t:opts/0"},{"type":"type","title":"ExRocketmq.Typespecs.properties/0","doc":"","ref":"ExRocketmq.Typespecs.html#t:properties/0"},{"type":"type","title":"ExRocketmq.Typespecs.req_code/0","doc":"","ref":"ExRocketmq.Typespecs.html#t:req_code/0"},{"type":"type","title":"ExRocketmq.Typespecs.resp_code/0","doc":"","ref":"ExRocketmq.Typespecs.html#t:resp_code/0"},{"type":"type","title":"ExRocketmq.Typespecs.str_dict/0","doc":"","ref":"ExRocketmq.Typespecs.html#t:str_dict/0"},{"type":"type","title":"ExRocketmq.Typespecs.sysflag/0","doc":"","ref":"ExRocketmq.Typespecs.html#t:sysflag/0"},{"type":"type","title":"ExRocketmq.Typespecs.topic/0","doc":"","ref":"ExRocketmq.Typespecs.html#t:topic/0"},{"type":"type","title":"ExRocketmq.Typespecs.transaction_id/0","doc":"","ref":"ExRocketmq.Typespecs.html#t:transaction_id/0"},{"type":"type","title":"ExRocketmq.Typespecs.transaction_state/0","doc":"","ref":"ExRocketmq.Typespecs.html#t:transaction_state/0"},{"type":"type","title":"ExRocketmq.Typespecs.transaction_type/0","doc":"","ref":"ExRocketmq.Typespecs.html#t:transaction_type/0"},{"type":"module","title":"ExRocketmq.Util.Assertion","doc":"断言工具","ref":"ExRocketmq.Util.Assertion.html"},{"type":"macro","title":"ExRocketmq.Util.Assertion.assert_exists/2","doc":"","ref":"ExRocketmq.Util.Assertion.html#assert_exists/2"},{"type":"macro","title":"ExRocketmq.Util.Assertion.assert_non_exists/2","doc":"","ref":"ExRocketmq.Util.Assertion.html#assert_non_exists/2"},{"type":"macro","title":"ExRocketmq.Util.Assertion.do_assert/2","doc":"","ref":"ExRocketmq.Util.Assertion.html#do_assert/2"},{"type":"module","title":"ExRocketmq.Util.BitHelper","doc":"bit tool for bit operation","ref":"ExRocketmq.Util.BitHelper.html"},{"type":"function","title":"ExRocketmq.Util.BitHelper.set_bit/3","doc":"","ref":"ExRocketmq.Util.BitHelper.html#set_bit/3"},{"type":"function","title":"ExRocketmq.Util.BitHelper.unset_bit/3","doc":"","ref":"ExRocketmq.Util.BitHelper.html#unset_bit/3"},{"type":"module","title":"ExRocketmq.Util.Bool","doc":"bool util","ref":"ExRocketmq.Util.Bool.html"},{"type":"function","title":"ExRocketmq.Util.Bool.boolean?/1","doc":"","ref":"ExRocketmq.Util.Bool.html#boolean?/1"},{"type":"module","title":"ExRocketmq.Util.Buffer","doc":"A simple buffer queue implementation using ETS tables.\n\nThis module provides a way to create a buffer queue with a specified capacity\nand allows items to be put into and taken out of the queue. If the queue is\nfull, `put` will return `{:error, :full}`. If the queue is empty, `take` will\nblock until an item is available.","ref":"ExRocketmq.Util.Buffer.html"},{"type":"module","title":"Examples - ExRocketmq.Util.Buffer","doc":"iex> {:ok, pid} = ExRocketmq.Util.Buffer.start_link(name: :my_queue, size: 10)\n    iex> ExRocketmq.Util.Buffer.put(:my_queue, [1, 2, 3])\n    :ok\n    iex> ExRocketmq.Util.Buffer.take(:my_queue)\n    [1, 2, 3]","ref":"ExRocketmq.Util.Buffer.html#module-examples"},{"type":"function","title":"ExRocketmq.Util.Buffer.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExRocketmq.Util.Buffer.html#child_spec/1"},{"type":"function","title":"ExRocketmq.Util.Buffer.first/2","doc":"Returns the first value in the buffer queue.","ref":"ExRocketmq.Util.Buffer.html#first/2"},{"type":"function","title":"Examples - ExRocketmq.Util.Buffer.first/2","doc":"iex> {:ok, _pid} = ExRocketmq.Util.Buffer.start_link(:my_queue, 10)\n    iex> ExRocketmq.Util.Buffer.put(:my_queue, [1, 2, 3])\n    :ok\n    iex> ExRocketmq.Util.Buffer.first(:my_queue)\n    1","ref":"ExRocketmq.Util.Buffer.html#first/2-examples"},{"type":"function","title":"ExRocketmq.Util.Buffer.last/2","doc":"Returns the last value in the buffer queue.","ref":"ExRocketmq.Util.Buffer.html#last/2"},{"type":"function","title":"Examples - ExRocketmq.Util.Buffer.last/2","doc":"iex> {:ok, _pid} = ExRocketmq.Util.Buffer.start_link(:my_queue, 10)\n    iex> ExRocketmq.Util.Buffer.put(:my_queue, [1, 2, 3])\n    :ok\n    iex> ExRocketmq.Util.Buffer.last(:my_queue)\n    3","ref":"ExRocketmq.Util.Buffer.html#last/2-examples"},{"type":"function","title":"ExRocketmq.Util.Buffer.put/3","doc":"Adds the given items to the buffer queue.\n\nIf the buffer queue is full, this function will return `{:error, :full}`.","ref":"ExRocketmq.Util.Buffer.html#put/3"},{"type":"function","title":"Examples - ExRocketmq.Util.Buffer.put/3","doc":"iex> {:ok, _pid} = ExRocketmq.Util.Buffer.start_link(:my_queue, 10)\n    iex> ExRocketmq.Util.Buffer.put(:my_queue, [1, 2, 3])\n    :ok\n    iex> ExRocketmq.Util.Buffer.put(:my_queue, [4, 5, 6], 1000)\n    :ok\n    iex> ExRocketmq.Util.Buffer.put(:my_queue, [7, 8, 9, 10, 11])\n    {:error, :full}","ref":"ExRocketmq.Util.Buffer.html#put/3-examples"},{"type":"function","title":"ExRocketmq.Util.Buffer.size/2","doc":"Returns the number of items in the buffer queue.","ref":"ExRocketmq.Util.Buffer.html#size/2"},{"type":"function","title":"Examples - ExRocketmq.Util.Buffer.size/2","doc":"iex> {:ok, _pid} = ExRocketmq.Util.Buffer.start_link(:my_queue, 10)\n    iex> ExRocketmq.Util.Buffer.put(:my_queue, [1, 2, 3])\n    :ok\n    iex> ExRocketmq.Util.Buffer.size(:my_queue)\n    3","ref":"ExRocketmq.Util.Buffer.html#size/2-examples"},{"type":"function","title":"ExRocketmq.Util.Buffer.start_link/1","doc":"Starts a new buffer process.","ref":"ExRocketmq.Util.Buffer.html#start_link/1"},{"type":"function","title":"Options - ExRocketmq.Util.Buffer.start_link/1","doc":"* `:name` - The name of the buffer process. Must be an atom\n* `:size` - The maximum number of items the buffer can hold","ref":"ExRocketmq.Util.Buffer.html#start_link/1-options"},{"type":"function","title":"Examples - ExRocketmq.Util.Buffer.start_link/1","doc":"iex> {:ok, pid} = ExRocketmq.Util.Buffer.start_link(name: :my_queue, size: 10)\n    iex> ExRocketmq.Util.Buffer.put(:my_queue, [1, 2, 3])","ref":"ExRocketmq.Util.Buffer.html#start_link/1-examples"},{"type":"function","title":"ExRocketmq.Util.Buffer.stop/1","doc":"","ref":"ExRocketmq.Util.Buffer.html#stop/1"},{"type":"function","title":"ExRocketmq.Util.Buffer.take/2","doc":"Removes and returns items from the buffer queue.\n\nIf the buffer queue is empty, this function will block until an item is available.","ref":"ExRocketmq.Util.Buffer.html#take/2"},{"type":"function","title":"Examples - ExRocketmq.Util.Buffer.take/2","doc":"iex> {:ok, _pid} = ExRocketmq.Util.Buffer.start_link(:my_queue, 10)\n    iex> ExRocketmq.Util.Buffer.put(:my_queue, [1, 2, 3])\n    :ok\n    iex> ExRocketmq.Util.Buffer.take(:my_queue)\n    [1, 2, 3]\n    iex> ExRocketmq.Util.Buffer.take(:my_queue)\n    []","ref":"ExRocketmq.Util.Buffer.html#take/2-examples"},{"type":"type","title":"ExRocketmq.Util.Buffer.t/0","doc":"","ref":"ExRocketmq.Util.Buffer.html#t:t/0"},{"type":"module","title":"ExRocketmq.Util.ClientId","doc":"generate client id","ref":"ExRocketmq.Util.ClientId.html"},{"type":"function","title":"ExRocketmq.Util.ClientId.get/1","doc":"","ref":"ExRocketmq.Util.ClientId.html#get/1"},{"type":"module","title":"ExRocketmq.Util.Const","doc":"a module to define constants\n```Elixir\ndefmodule MyApp.Constant do\n  import Const\n\n  const :facebook_url, \"http://facebook.com/rohanpujaris\"\n\nend\n\nrequire MyApp.Constant\n@facebook_url MyApp.Constant.facebook_url  # You can use this line anywhere to get the facebook url.\n```","ref":"ExRocketmq.Util.Const.html"},{"type":"macro","title":"ExRocketmq.Util.Const.const/2","doc":"","ref":"ExRocketmq.Util.Const.html#const/2"},{"type":"module","title":"ExRocketmq.Util.Debug","doc":"Debug tools","ref":"ExRocketmq.Util.Debug.html"},{"type":"function","title":"ExRocketmq.Util.Debug.debug/1","doc":"","ref":"ExRocketmq.Util.Debug.html#debug/1"},{"type":"function","title":"ExRocketmq.Util.Debug.stacktrace/1","doc":"","ref":"ExRocketmq.Util.Debug.html#stacktrace/1"},{"type":"module","title":"ExRocketmq.Util.Fnv1a","doc":"Hashes terms using the FNV-1a 32-bit algorithm.","ref":"ExRocketmq.Util.Fnv1a.html"},{"type":"module","title":"Examples - ExRocketmq.Util.Fnv1a","doc":"iex> ExRocketmq.Util.Fnv1a.hash({1, 2})\n    2761922969\n    iex> ExRocketmq.Util.Fnv1a.hash(\"this thing\")\n    891900385","ref":"ExRocketmq.Util.Fnv1a.html#module-examples"},{"type":"function","title":"ExRocketmq.Util.Fnv1a.hash/1","doc":"","ref":"ExRocketmq.Util.Fnv1a.html#hash/1"},{"type":"module","title":"ExRocketmq.Util.Hasher","doc":"Hash util","ref":"ExRocketmq.Util.Hasher.html"},{"type":"function","title":"ExRocketmq.Util.Hasher.hash_string/1","doc":"","ref":"ExRocketmq.Util.Hasher.html#hash_string/1"},{"type":"module","title":"ExRocketmq.Util.Network","doc":"network tools","ref":"ExRocketmq.Util.Network.html"},{"type":"function","title":"ExRocketmq.Util.Network.binary_to_ipv4/1","doc":"","ref":"ExRocketmq.Util.Network.html#binary_to_ipv4/1"},{"type":"function","title":"ExRocketmq.Util.Network.binary_to_ipv6/1","doc":"","ref":"ExRocketmq.Util.Network.html#binary_to_ipv6/1"},{"type":"function","title":"ExRocketmq.Util.Network.get_local_ipv4_address/0","doc":"","ref":"ExRocketmq.Util.Network.html#get_local_ipv4_address/0"},{"type":"function","title":"ExRocketmq.Util.Network.local_ip_addr/0","doc":"","ref":"ExRocketmq.Util.Network.html#local_ip_addr/0"},{"type":"function","title":"ExRocketmq.Util.Network.parse_addr/1","doc":"","ref":"ExRocketmq.Util.Network.html#parse_addr/1"},{"type":"module","title":"ExRocketmq.Util.Queue","doc":"a simple wrapper of fifo-queue with agent, so you don't have to keep queue in Genserver state, just a agent pid","ref":"ExRocketmq.Util.Queue.html"},{"type":"function","title":"ExRocketmq.Util.Queue.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExRocketmq.Util.Queue.html#child_spec/1"},{"type":"function","title":"ExRocketmq.Util.Queue.pop/1","doc":"pop a item from queue-agent","ref":"ExRocketmq.Util.Queue.html#pop/1"},{"type":"function","title":"ExRocketmq.Util.Queue.push/2","doc":"push a item to queue-agent","ref":"ExRocketmq.Util.Queue.html#push/2"},{"type":"function","title":"Examples - ExRocketmq.Util.Queue.push/2","doc":"iex> ExRocketmq.Util.Queue.push(agent, data)\n    {:ok, _}","ref":"ExRocketmq.Util.Queue.html#push/2-examples"},{"type":"function","title":"ExRocketmq.Util.Queue.start_link/1","doc":"start the queue agent","ref":"ExRocketmq.Util.Queue.html#start_link/1"},{"type":"function","title":"Examples - ExRocketmq.Util.Queue.start_link/1","doc":"iex> {:ok, agent} = ExRocketmq.Util.Queue.start_link()\n    {:ok, #PID }","ref":"ExRocketmq.Util.Queue.html#start_link/1-examples"},{"type":"function","title":"ExRocketmq.Util.Queue.stop/1","doc":"","ref":"ExRocketmq.Util.Queue.html#stop/1"},{"type":"module","title":"ExRocketmq.Util.Random","doc":"random kits","ref":"ExRocketmq.Util.Random.html"},{"type":"function","title":"ExRocketmq.Util.Random.generate_id/1","doc":"generate short random id","ref":"ExRocketmq.Util.Random.html#generate_id/1"},{"type":"function","title":"Examples - ExRocketmq.Util.Random.generate_id/1","doc":"iex> generate_id(\"H\")\n\"H68203790HX446F\"","ref":"ExRocketmq.Util.Random.html#generate_id/1-examples"},{"type":"module","title":"ExRocketmq.Util.UniqId","doc":"Utility module for generating unique IDs for RocketMQ messages or requests.\n\nA unique ID consists of two parts:\n- Prefix: IP address and process ID\n- Gap timestamp + counter\n\nThe prefix is composed of the IP address and process ID of the current process.\nThe gap timestamp is the difference between the current time and the start of the current month, in milliseconds.\nThe counter is an auto-incrementing number for each process.\n\nThis module uses an Agent to keep track of the state of the counter and timestamp.","ref":"ExRocketmq.Util.UniqId.html"},{"type":"module","title":"Examples - ExRocketmq.Util.UniqId","doc":"iex> ExRocketmq.Util.UniqId.get_uniq_id(pid)\n    \"0A93667D3B0A0000000063952A000001\"","ref":"ExRocketmq.Util.UniqId.html#module-examples"},{"type":"function","title":"ExRocketmq.Util.UniqId.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExRocketmq.Util.UniqId.html#child_spec/1"},{"type":"function","title":"ExRocketmq.Util.UniqId.get_uniq_id/1","doc":"generate a unique id","ref":"ExRocketmq.Util.UniqId.html#get_uniq_id/1"},{"type":"function","title":"Examples - ExRocketmq.Util.UniqId.get_uniq_id/1","doc":"iex> ExRocketmq.Util.UniqId.get_uniq_id(pid)\n    \"0A93667D3B0A0000000063952A000001\"","ref":"ExRocketmq.Util.UniqId.html#get_uniq_id/1-examples"},{"type":"function","title":"ExRocketmq.Util.UniqId.start_link/1","doc":"start a uniqid generater agent","ref":"ExRocketmq.Util.UniqId.html#start_link/1"},{"type":"function","title":"ExRocketmq.Util.UniqId.stop/1","doc":"","ref":"ExRocketmq.Util.UniqId.html#stop/1"},{"type":"exception","title":"ExRocketmq.Exception","doc":"General exception with an optional string, map, or Keyword list stored\nin exception details","ref":"ExRocketmq.Exception.html"},{"type":"function","title":"ExRocketmq.Exception.new/2","doc":"","ref":"ExRocketmq.Exception.html#new/2"}],"content_type":"text/markdown"}